#!/usr/bin/python
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_monitordatacollectionendpoint
version_added: "3.9.0"
short_description: Managed Data Collection Endpoints
description:
    - Create, update or delete the Data Collection Rules Endpoints.

options:
    name:
        description:
            - The name of the data collection endpoint.
            - The name is case insensitive.
        type: str
        required: true
    resource_group:
        description:
            - The name of the resource group in which the data collection endpoint is (if you use name)
        type: str
        required: true
    location:
        description:
            - The geo-location where the resource lives.
            - Default is resource group's location.
        type: str
    kind:
        description:
            - The kind of the resource.
        type: str
        choices:
            - Linux
            - Windows
    description:
        description:
            - Description of the data collection endpoint.
        type: str
    network_acls:
        description:
            - Network access control rules for the endpoints.
        type: dict
        suboptions:
            public_network_access:
                description:
                    - The configuration to set whether network access from public internet to the endpoints are allowed.
                type: str
                choices:
                    - Enabled
                    - Disabled
                    - SecuredByPerimeter
    state:
        description:
            - State of the data colleciton endpoint.
            - Set to C(present) to create a new endpoint.
            - Set to C(absent) to remove a endpoint.
        default: present
        type: str
        choices:
            - absent
            - present

extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags

author:
    - magodo (@magodo)
    - Fred Sun (@Fred-sun)
'''

EXAMPLES = '''
- name: Create a new data collection endpoint
  azure.azcollection.azure_rm_monitordatacollectionendpoint:
    resource_group: v-xisuRG
    name: fredendpoint1
    description: fredtestend
    kind: Linux
    network_acls:
      public_network_access: Enabled
    tags:
      key1: value1

- name: Delete the data collection endpoint
  azure.azcollection.azure_rm_monitordatacollectionendpoint:
    resource_group: v-xisuRG
    name: fredendpoint1
    state: absent
'''

RETURN = '''
datacollectionendpoint:
    description:
        - The facts of data collection rule association.
    type: complex
    returned: always
    contains:
        configuration_access:
            description:
                - The endpoint used by clients to access their configuration.
            type: dict
            returned: always
            sample: {"endpoint": "https://fredendpoint-q7lu.eastus-1.handler.control.monitor.azure.com"}
        description:
            description:
                - Description of the data collection endpoint.
            type: str
            returned: always
            sample: Created
        etag:
            description:
                - Resource entity tag (ETag).
            type: str
            returned: always
            sample: "3d001f14-0000-0100-0000-68ca270a0000"
        id:
            description:
                - Fully qualified ID of the resource.
            type: str
            returned: always
            sample: "/subscriptions/xxx-xxx/resourceGroups/v-xisuRG/providers/Microsoft.Insights/dataCollectionEndpoints/fredendpoint"
        immutable_id:
            description:
                - The immutable ID of this data collection endpoint resource.
                - This property is READ-ONLY.
            type: str
            returned: always
            sample: dce-9897a7cde9b54676a1c07ab3ea222768
        kind:
            description:
                - The kind of the resource.
            type: str
            returned: always
            sample: Linux
        location:
            description:
                - The geo-location where the resource lives.
            type: str
            returned: always
            sample: eastus
        logs_ingestion:
            description:
                - The endpoint used by clients to ingest logs.
            type: dict
            returned: always
            sample: {"endpoint": "https://fredendpoint-q7lu.eastus-1.ingest.monitor.azure.com"}
        metrics_ingestion:
            description:
                - The endpoint used by clients to ingest metrics.
            type: dict
            returned: always
            sample: {"endpoint": "https://fredendpoint-q7lu.eastus-1.metrics.ingest.monitor.azure.com"}
        name:
            description:
                - The name of the resource.
            type: str
            returned: always
            sample: testendpoint
        network_acls:
            description:
                - Network access control rules for the endpoints.
            type: dict
            returned: always
            sample: {"public_network_access": "Enabled"}
        provisioning_state:
            description:
                - The resource provisioning state. This property is READ-ONLY.
            type: str
            returned: always
            sample: Succeeded
        system_data:
            description:
                - Metadata pertaining to creation and last modification of the resource.
            type: dict
            returned: always
            sample: {
                "created_at": "2025-09-17T03:12:08.743499Z",
                "created_by": "00867800-0fa3-4d02-8bc8-35edac3a0d32",
                "created_by_type": "Application",
                "last_modified_at": "2025-09-17T03:12:08.743499Z",
                "last_modified_by": "00867800-0fa3-4d02-8bc8-35edac3a0d32",
                "last_modified_by_type": "Application"
            }
        tags:
            description:
                - Resource tags.
            returned: always
            type: dict
            sample: {'key1': 'value1'}
        type:
            description:
                - The type of the resource.
            returned: always
            type: str
            sample: "Microsoft.Insights/dataCollectionEndpoints"
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ext import AzureRMModuleBaseExt


network_acls_spec = dict(
    public_network_access=dict(
        type='str',
        choices=["Enabled", "Disabled", "SecuredByPerimeter"]
    )
)


class AzureRMDataCollectionRuleEndpoint(AzureRMModuleBaseExt):
    """Information class for an Azure RM Data Collection Rules"""

    def __init__(self):
        self.module_arg_spec = dict(
            name=dict(type='str', required=True),
            resource_group=dict(type='str', required=True),
            location=dict(type='str'),
            kind=dict(type='str', choices=['Linux', 'Windows']),
            description=dict(type='str'),
            state=dict(type='str', default='present', choices=['present', 'absent']),
            network_acls=dict(
                type='dict',
                options=network_acls_spec
            )
        )

        self.resource_group = None
        self.name = None
        self.location = None
        self.kind = None
        self.description = None
        self.network_acls = None
        self.tags = None
        self.state = None
        self.log_path = None
        self.log_mode = None
        self.body = dict()

        self.results = dict(
            changed=False,
            datacollectionendpoint=None
        )

        super(AzureRMDataCollectionRuleEndpoint, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                                supports_check_mode=True,
                                                                supports_tags=True,
                                                                facts_module=True)

    def exec_module(self, **kwargs):
        """Main module execution method"""

        for key in list(self.module_arg_spec.keys()) + ['tags']:
            if key in ['resource_group', 'state', 'name']:
                setattr(self, key, kwargs[key])
            else:
                self.body[key] = kwargs[key]

        resource_group = self.get_resource_group(self.resource_group)
        if not self.body.get('location'):
            # Set default location
            self.body['location'] = resource_group.location

        self.tags = self.body.get('tags')
        response = self.get_endpoint()
        changed = False
        if self.state == 'present':
            if response:
                self.log("The monitor data collection endpoint already exist")
                update_tags, self.body['tags'] = self.update_tags(response.get('tags', dict()))
                if update_tags:
                    changed = True
                if self.body.get('description') and self.body['description'] != response.get('description'):
                    changed = True
                else:
                    self.body['description'] = response.get('description')
                if self.body.get('kind') and self.body.get('kind') != response.get('kind'):
                    changed = True
                else:
                    self.body['kind'] = response.get('kind')
                if self.body.get('network_acls') and self.body['network_acls'] != response.get('network_acls'):
                    changed = True
                else:
                    self.body['network_acls'] = response.get('network_acls')
            else:
                changed = True
                self.log("There is no monitor data collection endpoint, will create a new")

            if self.check_mode:
                self.log("Check mode test")
            elif changed:
                response = self.create_endpoint(self.body)
        else:
            if response:
                changed = True
                if self.check_mode:
                    self.log("The monitor data collection endpoint already exist, will be delete")
                else:
                    response = self.delete_endpoint()
            else:
                if self.check_mode:
                    self.log("There is no monitor data collection endpoint.")

        self.results['datacollectionruleendpoint'] = response
        self.results['changed'] = changed
        return self.results

    def get_endpoint(self):
        '''
        Gets the specified data collection rule endpoint
        '''
        response = None
        try:
            response = self.monitor_management_client_data_collection_rules.data_collection_endpoints.get(resource_group_name=self.resource_group,
                                                                                                          data_collection_endpoint_name=self.name)
        except Exception as ex:
            self.log("Could not find data collection endpoint {0} in resource group {1}, Exception as {2}".format(self.name, self.resource_group, ex))
        if response:
            return response.as_dict()

    def create_endpoint(self, body):
        '''
        Create a new Data Collection Endpoint
        '''
        response = None
        try:
            response = self.monitor_management_client_data_collection_rules.data_collection_endpoints.create(resource_group_name=self.resource_group,
                                                                                                             data_collection_endpoint_name=self.name,
                                                                                                             body=body)
        except Exception as ex:
            self.fail("Create the data collection endponts occurred exception, Exception as {0}".format(ex))

        if response:
            return response.as_dict()

    def delete_endpoint(self):
        '''
        Delete the Data Collection Endpoint
        '''
        try:
            self.monitor_management_client_data_collection_rules.data_collection_endpoints.delete(resource_group_name=self.resource_group,
                                                                                                  data_collection_endpoint_name=self.name)
        except Exception as ex:
            self.fail("Delete the data collection endponts occurred exception, Exception as {0}".format(ex))


def main():
    """Main execution"""
    AzureRMDataCollectionRuleEndpoint()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: networks_wireless_rf_profiles_info
short_description: Information module for networks _wireless _rf _profiles
description:
  - Get all networks _wireless _rf _profiles.
  - Get networks _wireless _rf _profiles by id.
  - List RF profiles for this network.
  - Return a RF profile.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  networkId:
    description:
      - NetworkId path parameter. Network ID.
    type: str
  includeTemplateProfiles:
    description:
      - >
        IncludeTemplateProfiles query parameter. If the network is bound to a template,
        this parameter controls whether or not the non-basic RF profiles defined on
        the template should be included in the response alongside the non-basic profiles
        defined on the bound network. Defaults to false.
    type: bool
  rfProfileId:
    description:
      - RfProfileId path parameter. Rf profile ID.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for wireless getNetworkWirelessRfProfile
    description: Complete reference of the getNetworkWirelessRfProfile API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-network-wireless-rf-profile
  - name: Cisco Meraki documentation for wireless getNetworkWirelessRfProfiles
    description: Complete reference of the getNetworkWirelessRfProfiles API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-network-wireless-rf-profiles
notes:
  - SDK Method used are
    wireless.Wireless.get_network_wireless_rf_profile,
    wireless.Wireless.get_network_wireless_rf_profiles,
  - Paths used are
    get /networks/{networkId}/wireless/rfProfiles,
    get /networks/{networkId}/wireless/rfProfiles/{rfProfileId},
"""

EXAMPLES = r"""
- name: Get all networks _wireless _rf _profiles
  cisco.meraki.networks_wireless_rf_profiles_info:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    includeTemplateProfiles: true
    networkId: string
  register: result
- name: Get networks _wireless _rf _profiles by id
  cisco.meraki.networks_wireless_rf_profiles_info:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    networkId: string
    rfProfileId: string
  register: result
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "id": "string",
      "networkId": "string",
      "name": "string",
      "clientBalancingEnabled": true,
      "minBitrateType": "string",
      "bandSelectionType": "string",
      "apBandSettings": {
        "bandOperationMode": "string",
        "bands": {
          "enabled": [
            "string"
          ]
        },
        "bandSteeringEnabled": true
      },
      "twoFourGhzSettings": {
        "maxPower": 0,
        "minPower": 0,
        "minBitrate": 0,
        "validAutoChannels": [
          0
        ],
        "axEnabled": true,
        "rxsop": 0
      },
      "fiveGhzSettings": {
        "maxPower": 0,
        "minPower": 0,
        "minBitrate": 0,
        "validAutoChannels": [
          0
        ],
        "channelWidth": "string",
        "rxsop": 0
      },
      "sixGhzSettings": {
        "maxPower": 0,
        "minPower": 0,
        "minBitrate": 0,
        "validAutoChannels": [
          0
        ],
        "channelWidth": "string",
        "rxsop": 0
      },
      "transmission": {
        "enabled": true
      },
      "perSsidSettings": {
        "0": {
          "name": "string",
          "minBitrate": 0,
          "bandOperationMode": "string",
          "bands": {
            "enabled": [
              "string"
            ]
          },
          "bandSteeringEnabled": true
        },
        "1": {
          "name": "string",
          "minBitrate": 0,
          "bandOperationMode": "string",
          "bands": {
            "enabled": [
              "string"
            ]
          },
          "bandSteeringEnabled": true
        },
        "2": {
          "name": "string",
          "minBitrate": 0,
          "bandOperationMode": "string",
          "bands": {
            "enabled": [
              "string"
            ]
          },
          "bandSteeringEnabled": true
        },
        "3": {
          "name": "string",
          "minBitrate": 0,
          "bandOperationMode": "string",
          "bands": {
            "enabled": [
              "string"
            ]
          },
          "bandSteeringEnabled": true
        },
        "4": {
          "name": "string",
          "minBitrate": 0,
          "bandOperationMode": "string",
          "bands": {
            "enabled": [
              "string"
            ]
          },
          "bandSteeringEnabled": true
        },
        "5": {
          "name": "string",
          "minBitrate": 0,
          "bandOperationMode": "string",
          "bands": {
            "enabled": [
              "string"
            ]
          },
          "bandSteeringEnabled": true
        },
        "6": {
          "name": "string",
          "minBitrate": 0,
          "bandOperationMode": "string",
          "bands": {
            "enabled": [
              "string"
            ]
          },
          "bandSteeringEnabled": true
        },
        "7": {
          "name": "string",
          "minBitrate": 0,
          "bandOperationMode": "string",
          "bands": {
            "enabled": [
              "string"
            ]
          },
          "bandSteeringEnabled": true
        },
        "8": {
          "name": "string",
          "minBitrate": 0,
          "bandOperationMode": "string",
          "bands": {
            "enabled": [
              "string"
            ]
          },
          "bandSteeringEnabled": true
        },
        "9": {
          "name": "string",
          "minBitrate": 0,
          "bandOperationMode": "string",
          "bands": {
            "enabled": [
              "string"
            ]
          },
          "bandSteeringEnabled": true
        },
        "10": {
          "name": "string",
          "minBitrate": 0,
          "bandOperationMode": "string",
          "bands": {
            "enabled": [
              "string"
            ]
          },
          "bandSteeringEnabled": true
        },
        "11": {
          "name": "string",
          "minBitrate": 0,
          "bandOperationMode": "string",
          "bands": {
            "enabled": [
              "string"
            ]
          },
          "bandSteeringEnabled": true
        },
        "12": {
          "name": "string",
          "minBitrate": 0,
          "bandOperationMode": "string",
          "bands": {
            "enabled": [
              "string"
            ]
          },
          "bandSteeringEnabled": true
        },
        "13": {
          "name": "string",
          "minBitrate": 0,
          "bandOperationMode": "string",
          "bands": {
            "enabled": [
              "string"
            ]
          },
          "bandSteeringEnabled": true
        },
        "14": {
          "name": "string",
          "minBitrate": 0,
          "bandOperationMode": "string",
          "bands": {
            "enabled": [
              "string"
            ]
          },
          "bandSteeringEnabled": true
        }
      },
      "isIndoorDefault": true,
      "isOutdoorDefault": true
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: networks_bind
short_description: Resource module for networks _bind
description:
  - Manage operation create of the resource networks _bind.
  - Bind a network to a template.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  autoBind:
    description: Optional boolean indicating whether the network's switches should
      automatically bind to profiles of the same model. Defaults to false if left
      unspecified. This option only affects switch networks and switch templates.
      Auto-bind is not valid unless the switch template has at least one profile and
      has at most one profile per switch model.
    type: bool
  configTemplateId:
    description: The ID of the template to which the network should be bound.
    type: str
  networkId:
    description: NetworkId path parameter. Network ID.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for networks bindNetwork
    description: Complete reference of the bindNetwork API.
    link: https://developer.cisco.com/meraki/api-v1/#!bind-network
notes:
  - SDK Method used are
    networks.Networks.bind_network,
  - Paths used are
    post /networks/{networkId}/bind,
"""

EXAMPLES = r"""
- name: Create
  cisco.meraki.networks_bind:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    autoBind: false
    configTemplateId: N_23952905
    networkId: string
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "id": "string",
      "organizationId": "string",
      "name": "string",
      "productTypes": [
        "string"
      ],
      "timeZone": "string",
      "tags": [
        "string"
      ],
      "enrollmentString": "string",
      "url": "string",
      "notes": "string",
      "isBoundToConfigTemplate": true,
      "configTemplateId": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2018, Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: iam_user_info
version_added: 5.0.0
short_description: Gather IAM user(s) facts in AWS
description:
  - This module can be used to gather IAM user(s) facts in AWS.
  - This module was originally added to C(community.aws) in release 1.0.0.
author:
  - Constantin Bugneac (@Constantin07)
  - Abhijeet Kasurde (@Akasurde)
options:
  name:
    description:
      - The name of the IAM user to look for.
      - O(user_name) was added as an alias in release 7.2.0.
    required: false
    type: str
    aliases: ["user_name"]
  group:
    description:
      - The group name name of the IAM user to look for. Mutually exclusive with C(path).
      - O(group_name) was added as an alias in release 7.2.0.
    required: false
    type: str
    aliases: ["group_name"]
  path_prefix:
    description:
      - The path to the IAM user. Mutually exclusive with O(group).
      - If specified, then would get all user names whose path starts with user provided value.
    required: false
    default: '/'
    type: str
    aliases: ["path", "prefix"]
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# Note: These examples do not set authentication details, see the AWS Guide for details.
# Gather facts about "test" user.
- name: Get IAM user info
  amazon.aws.iam_user_info:
    name: "test"

# Gather facts about all users in the "dev" group.
- name: Get IAM user info
  amazon.aws.iam_user_info:
    group: "dev"

# Gather facts about all users with "/division_abc/subdivision_xyz/" path.
- name: Get IAM user info
  amazon.aws.iam_user_info:
    path: "/division_abc/subdivision_xyz/"
"""

RETURN = r"""
iam_users:
    description: List of maching iam users.
    returned: always
    type: complex
    contains:
        arn:
            description: The ARN of the user.
            returned: when user exists.
            type: str
            sample: "arn:aws:iam::123456789012:user/dev/test_user"
        create_date:
            description: The datetime user was created.
            returned: when user exists.
            type: str
            sample: "2016-05-24T12:24:59+00:00"
        password_last_used:
            description: The last datetime the password was used by user.
            returned: if password was used at least once
            type: str
            sample: "2016-05-25T13:39:11+00:00"
        path:
            description: The path to user.
            returned: when user exists.
            type: str
            sample: "/dev/"
        user_id:
            description: The unique user id.
            returned: when user exists.
            type: str
            sample: "AIDUIOOCQKTUGI6QJLGH2"
        user_name:
            description: The user name.
            returned: when user exists.
            type: str
            sample: "test_user"
        tags:
            description: User tags.
            type: dict
            returned: when user exists.
            sample: '{"Env": "Prod"}'
        login_profile:
            description:
              - Detailed login profile information if the user has access to log in from AWS default console.
              - Returns an empty object {} if no access.
            returned: when user exists.
            type: dict
            sample: {"create_date": "2024-03-20T12:50:56+00:00", "password_reset_required": false, "user_name": "i_am_a_user"}
            contains:
              create_date:
                description: The date and time, in ISO 8601 date-time format, when the user was created.
                type: str
                returned: always
              password_reset_required:
                description: Specifies whether the user is required to set a new password on next sign-in.
                type: bool
                returned: always
              user_name:
                description: The name of the IAM user
                type: str
                returned: always
"""

from ansible_collections.amazon.aws.plugins.module_utils.iam import AnsibleIAMError
from ansible_collections.amazon.aws.plugins.module_utils.iam import IAMErrorHandler
from ansible_collections.amazon.aws.plugins.module_utils.iam import get_iam_group
from ansible_collections.amazon.aws.plugins.module_utils.iam import get_iam_user
from ansible_collections.amazon.aws.plugins.module_utils.iam import list_iam_user_tags
from ansible_collections.amazon.aws.plugins.module_utils.iam import list_iam_users
from ansible_collections.amazon.aws.plugins.module_utils.iam import normalize_iam_user
from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry


@IAMErrorHandler.list_error_handler("get login profile", {})
@AWSRetry.jittered_backoff()
def check_console_access(connection, user_name):
    return connection.get_login_profile(UserName=user_name)["LoginProfile"]


def _list_users(connection, name, group, path):
    # name but not path or group
    if name and path == "/" and not group:
        return [get_iam_user(connection, name, normalize=False)]

    iam_users = []

    if group:
        iam_group = get_iam_group(connection, group)
        if iam_group:
            iam_users = iam_group["Users"]
    else:
        iam_users = list_iam_users(connection, path=path)

    if not iam_users:
        return []

    # filter by name when a path or group was specified
    if name:
        iam_users = [u for u in iam_users if u["UserName"] == name]

    for user in iam_users:
        user["Tags"] = list_iam_user_tags(connection, user["UserName"])

    return iam_users


def list_users(connection, name, group, path):
    users = _list_users(connection, name, group, path)
    users = [u for u in users if u is not None]
    for user in users:
        user["LoginProfile"] = check_console_access(connection, user["UserName"])
    return [normalize_iam_user(user) for user in users]


def main():
    argument_spec = dict(
        name=dict(aliases=["user_name"]),
        group=dict(aliases=["group_name"]),
        path_prefix=dict(aliases=["path", "prefix"], default="/"),
    )

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        mutually_exclusive=[["group", "path_prefix"]],
        supports_check_mode=True,
    )

    name = module.params.get("name")
    group = module.params.get("group")
    path = module.params.get("path_prefix")

    connection = module.client("iam")

    try:
        module.exit_json(changed=False, iam_users=list_users(connection, name, group, path))
    except AnsibleIAMError as e:
        module.fail_json_aws_error(e)


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (c) 2019 Piotr Wojciechowski <piotr@it-playground.pl>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: docker_node
short_description: Manage Docker Swarm node
description:
  - Manages the Docker nodes through a Swarm Manager.
  - This module allows to change the node's role, its availability, and to modify, add or remove node labels.
extends_documentation_fragment:
  - community.docker._docker
  - community.docker._docker.docker_py_2_documentation
  - community.docker._attributes
  - community.docker._attributes.actiongroup_docker

attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
  idempotent:
    support: full

options:
  hostname:
    description:
      - The hostname or ID of node as registered in Swarm.
      - If more than one node is registered using the same hostname the ID must be used, otherwise module will fail.
    type: str
    required: true
  labels:
    description:
      - User-defined key/value metadata that will be assigned as node attribute.
      - Label operations in this module apply to the docker swarm node specified by O(hostname). Use M(community.docker.docker_swarm)
        module to add/modify/remove swarm cluster labels.
      - The actual state of labels assigned to the node when module completes its work depends on O(labels_state) and O(labels_to_remove)
        parameters values. See description below.
    type: dict
  labels_state:
    description:
      - It defines the operation on the labels assigned to node and labels specified in O(labels) option.
      - Set to V(merge) to combine labels provided in O(labels) with those already assigned to the node. If no labels are
        assigned then it will add listed labels. For labels that are already assigned to the node, it will update their values.
        The labels not specified in O(labels) will remain unchanged. If O(labels) is empty then no changes will be made.
      - Set to V(replace) to replace all assigned labels with provided ones. If O(labels) is empty then all labels assigned
        to the node will be removed.
    type: str
    default: 'merge'
    choices:
      - merge
      - replace
  labels_to_remove:
    description:
      - List of labels that will be removed from the node configuration. The list has to contain only label names, not their
        values.
      - If the label provided on the list is not assigned to the node, the entry is ignored.
      - If the label is both on the O(labels_to_remove) and O(labels), then value provided in O(labels) remains assigned to
        the node.
      - If O(labels_state=replace) and O(labels) is not provided or empty then all labels assigned to node are removed and
        O(labels_to_remove) is ignored.
    type: list
    elements: str
  availability:
    description: Node availability to assign. If not provided then node availability remains unchanged.
    choices:
      - active
      - pause
      - drain
    type: str
  role:
    description: Node role to assign. If not provided then node role remains unchanged.
    choices:
      - manager
      - worker
    type: str

requirements:
  - "L(Docker SDK for Python,https://docker-py.readthedocs.io/en/stable/) >= 2.4.0"
  - Docker API >= 1.25
author:
  - Piotr Wojciechowski (@WojciechowskiPiotr)
  - Thierry Bouvet (@tbouvet)
"""

EXAMPLES = r"""
---
- name: Set node role
  community.docker.docker_node:
    hostname: mynode
    role: manager

- name: Set node availability
  community.docker.docker_node:
    hostname: mynode
    availability: drain

- name: Replace node labels with new labels
  community.docker.docker_node:
    hostname: mynode
    labels:
      key: value
    labels_state: replace

- name: Merge node labels and new labels
  community.docker.docker_node:
    hostname: mynode
    labels:
      key: value

- name: Remove all labels assigned to node
  community.docker.docker_node:
    hostname: mynode
    labels_state: replace

- name: Remove selected labels from the node
  community.docker.docker_node:
    hostname: mynode
    labels_to_remove:
      - key1
      - key2
"""

RETURN = r"""
node:
  description: Information about node after 'update' operation.
  returned: success
  type: dict
"""

import traceback
import typing as t

try:
    from docker.errors import APIError, DockerException
except ImportError:
    # missing Docker SDK for Python handled in ansible.module_utils.docker.common
    pass

from ansible.module_utils.common.text.converters import to_text

from ansible_collections.community.docker.plugins.module_utils._common import (
    RequestException,
)
from ansible_collections.community.docker.plugins.module_utils._swarm import (
    AnsibleDockerSwarmClient,
)
from ansible_collections.community.docker.plugins.module_utils._util import (
    DockerBaseClass,
    sanitize_labels,
)


class TaskParameters(DockerBaseClass):
    hostname: str

    def __init__(self, client: AnsibleDockerSwarmClient) -> None:
        super().__init__()

        # Spec
        self.labels: dict[str, t.Any] | None = None
        self.labels_state: t.Literal["merge", "replace"] = "merge"
        self.labels_to_remove: list[str] | None = None

        # Node
        self.availability: t.Literal["active", "pause", "drain"] | None = None
        self.role: t.Literal["worker", "manager"] | None = None

        for key, value in client.module.params.items():
            setattr(self, key, value)

        sanitize_labels(self.labels, "labels", client)


class SwarmNodeManager(DockerBaseClass):
    def __init__(
        self, client: AnsibleDockerSwarmClient, results: dict[str, t.Any]
    ) -> None:
        super().__init__()

        self.client = client
        self.results = results
        self.check_mode = self.client.check_mode

        self.client.fail_task_if_not_swarm_manager()

        self.parameters = TaskParameters(client)

        self.node_update()

    def node_update(self) -> None:
        if not (self.client.check_if_swarm_node(node_id=self.parameters.hostname)):
            self.client.fail("This node is not part of a swarm.")

        if self.client.check_if_swarm_node_is_down():
            self.client.fail("Can not update the node. The node is down.")

        try:
            node_info = self.client.inspect_node(node_id=self.parameters.hostname)
        except APIError as exc:
            self.client.fail(f"Failed to get node information for {exc}")

        changed = False
        node_spec: dict[str, t.Any] = {
            "Availability": self.parameters.availability,
            "Role": self.parameters.role,
            "Labels": self.parameters.labels,
        }

        if self.parameters.role is None:
            node_spec["Role"] = node_info["Spec"]["Role"]
        else:
            if node_info["Spec"]["Role"] != self.parameters.role:
                node_spec["Role"] = self.parameters.role
                changed = True

        if self.parameters.availability is None:
            node_spec["Availability"] = node_info["Spec"]["Availability"]
        else:
            if node_info["Spec"]["Availability"] != self.parameters.availability:
                node_info["Spec"]["Availability"] = self.parameters.availability
                changed = True

        if self.parameters.labels_state == "replace":
            if self.parameters.labels is None:
                node_spec["Labels"] = {}
                if node_info["Spec"]["Labels"]:
                    changed = True
            else:
                if (node_info["Spec"]["Labels"] or {}) != self.parameters.labels:
                    node_spec["Labels"] = self.parameters.labels
                    changed = True
        elif self.parameters.labels_state == "merge":
            labels: dict[str, str] = dict(node_info["Spec"]["Labels"] or {})
            node_spec["Labels"] = labels
            if self.parameters.labels is not None:
                for key, value in self.parameters.labels.items():
                    if labels.get(key) != value:
                        labels[key] = value
                        changed = True

            if self.parameters.labels_to_remove is not None:
                for key in self.parameters.labels_to_remove:
                    if self.parameters.labels is not None:
                        if not self.parameters.labels.get(key):
                            if node_spec["Labels"].get(key):
                                node_spec["Labels"].pop(key)
                                changed = True
                        else:
                            self.client.module.warn(
                                f"Label '{to_text(key)}' listed both in 'labels' and 'labels_to_remove'. "
                                "Keeping the assigned label value."
                            )
                    else:
                        if node_spec["Labels"].get(key):
                            node_spec["Labels"].pop(key)
                            changed = True

        if changed is True:
            if not self.check_mode:
                try:
                    self.client.update_node(
                        node_id=node_info["ID"],
                        version=node_info["Version"]["Index"],
                        node_spec=node_spec,
                    )
                except APIError as exc:
                    self.client.fail(f"Failed to update node : {exc}")
            self.results["node"] = self.client.get_node_inspect(node_id=node_info["ID"])
            self.results["changed"] = changed
        else:
            self.results["node"] = node_info
            self.results["changed"] = changed


def main() -> None:
    argument_spec = {
        "hostname": {"type": "str", "required": True},
        "labels": {"type": "dict"},
        "labels_state": {
            "type": "str",
            "default": "merge",
            "choices": ["merge", "replace"],
        },
        "labels_to_remove": {"type": "list", "elements": "str"},
        "availability": {"type": "str", "choices": ["active", "pause", "drain"]},
        "role": {"type": "str", "choices": ["worker", "manager"]},
    }

    client = AnsibleDockerSwarmClient(
        argument_spec=argument_spec,
        supports_check_mode=True,
        min_docker_version="2.4.0",
    )

    try:
        results = {
            "changed": False,
        }

        SwarmNodeManager(client, results)
        client.module.exit_json(**results)
    except DockerException as e:
        client.fail(
            f"An unexpected Docker error occurred: {e}",
            exception=traceback.format_exc(),
        )
    except RequestException as e:
        client.fail(
            f"An unexpected requests error occurred when Docker SDK for Python tried to talk to the docker daemon: {e}",
            exception=traceback.format_exc(),
        )


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2013, Ivan Vanderbyl <ivan@app.io>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: logentries
author: "Ivan Vanderbyl (@ivanvanderbyl)"
short_description: Module for tracking logs using U(logentries.com)
description:
  - Sends logs to LogEntries in realtime.
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  path:
    type: str
    description:
      - Path to a log file.
    required: true
  state:
    type: str
    description:
      - Following state of the log.
    choices: ['present', 'absent', 'followed', 'unfollowed']
    default: present
  name:
    type: str
    description:
      - Name of the log.
  logtype:
    type: str
    description:
      - Type of the log.
    aliases: [type]

notes:
  - Requires the LogEntries agent which can be installed following the instructions at U(logentries.com).
"""

EXAMPLES = r"""
- name: Track nginx logs
  community.general.logentries:
    path: /var/log/nginx/access.log
    state: present
    name: nginx-access-log

- name: Stop tracking nginx logs
  community.general.logentries:
    path: /var/log/nginx/error.log
    state: absent
"""

from ansible.module_utils.basic import AnsibleModule


def query_log_status(module, le_path, path, state="present"):
    """Returns whether a log is followed or not."""

    if state == "present":
        rc, out, err = module.run_command([le_path, "followed", path])
        return rc == 0


def follow_log(module, le_path, logs, name=None, logtype=None):
    """Follows one or more logs if not already followed."""

    followed_count = 0

    for log in logs:
        if query_log_status(module, le_path, log):
            continue

        if module.check_mode:
            module.exit_json(changed=True)

        cmd = [le_path, "follow", log]
        if name:
            cmd.extend(["--name", name])
        if logtype:
            cmd.extend(["--type", logtype])
        rc, out, err = module.run_command(cmd)

        if not query_log_status(module, le_path, log):
            module.fail_json(msg=f"failed to follow '{log}': {err.strip()}")

        followed_count += 1

    if followed_count > 0:
        module.exit_json(changed=True, msg=f"followed {followed_count} log(s)")

    module.exit_json(changed=False, msg="logs(s) already followed")


def unfollow_log(module, le_path, logs):
    """Unfollows one or more logs if followed."""

    removed_count = 0

    # Using a for loop in case of error, we can report the package that failed
    for log in logs:
        # Query the log first, to see if we even need to remove.
        if not query_log_status(module, le_path, log):
            continue

        if module.check_mode:
            module.exit_json(changed=True)
        rc, out, err = module.run_command([le_path, "rm", log])

        if query_log_status(module, le_path, log):
            module.fail_json(msg=f"failed to remove '{log}': {err.strip()}")

        removed_count += 1

    if removed_count > 0:
        module.exit_json(changed=True, msg=f"removed {removed_count} package(s)")

    module.exit_json(changed=False, msg="logs(s) already unfollowed")


def main():
    module = AnsibleModule(
        argument_spec=dict(
            path=dict(required=True),
            state=dict(default="present", choices=["present", "followed", "absent", "unfollowed"]),
            name=dict(type="str"),
            logtype=dict(type="str", aliases=["type"]),
        ),
        supports_check_mode=True,
    )

    le_path = module.get_bin_path("le", True, ["/usr/local/bin"])

    p = module.params

    # Handle multiple log files
    logs = p["path"].split(",")
    logs = [_f for _f in logs if _f]

    if p["state"] in ["present", "followed"]:
        follow_log(module, le_path, logs, name=p["name"], logtype=p["logtype"])

    elif p["state"] in ["absent", "unfollowed"]:
        unfollow_log(module, le_path, logs)


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# Copyright (c) Ansible project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = """
name: loganalytics_ingestion
type: notification
short_description: Posts task results to an Azure Log Analytics workspace using the new Logs Ingestion API
author:
  - Wade Cline (@wtcline-intc) <wade.cline@intel.com>
  - Sriramoju Vishal Bharath (@vsh47) <sriramoju.vishal.bharath@intel.com>
  - Cyrus Li (@zhcli) <cyrus1006@gmail.com>
description:
  - This callback plugin will post task results in JSON format to an Azure Log Analytics workspace using the new Logs Ingestion API.
version_added: "12.4.0"
requirements:
  - The callback plugin has been enabled.
  - An Azure Log Analytics workspace has been established.
  - A Data Collection Rule (DCR) and custom table are created.
options:
  dce_url:
    description: URL of the Data Collection Endpoint (DCE) for Azure Logs Ingestion API.
    type: str
    required: true
    env:
      - name: ANSIBLE_LOGANALYTICS_DCE_URL
    ini:
      - section: callback_loganalytics
        key: dce_url
  dcr_id:
    description: Data Collection Rule (DCR) ID for the Azure Log Ingestion API.
    type: str
    required: true
    env:
      - name: ANSIBLE_LOGANALYTICS_DCR_ID
    ini:
      - section: callback_loganalytics
        key: dcr_id
  disable_attempts:
    description:
      - When O(disable_on_failure=true), number of plugin failures that must occur before the plugin is disabled.
      - This helps prevent outright plugin failure from a single, transient network issue.
    type: int
    default: 3
    env:
      - name: ANSIBLE_LOGANALYTICS_DISABLE_ATTEMPTS
    ini:
      - section: callback_loganalytics
        key: disable_attempts
  disable_on_failure:
    description: Stop trying to send data on plugin failure.
    type: bool
    default: true
    env:
      - name: ANSIBLE_LOGANALYTICS_DISABLE_ON_FAILURE
    ini:
      - section: callback_loganalytics
        key: disable_on_failure
  client_id:
    description: Client ID of the Azure App registration for OAuth2 authentication ("Modern Authentication").
    type: str
    required: true
    env:
      - name: ANSIBLE_LOGANALYTICS_CLIENT_ID
    ini:
      - section: callback_loganalytics
        key: client_id
  client_secret:
    description: Client Secret of the Azure App registration.
    type: str
    required: true
    env:
      - name: ANSIBLE_LOGANALYTICS_CLIENT_SECRET
    ini:
      - section: callback_loganalytics
        key: client_secret
  include_content:
    description: Send the content to the Azure Log Analytics workspace.
    type: bool
    default: false
    env:
      - name: ANSIBLE_LOGANALYTICS_INCLUDE_CONTENT
    ini:
      - section: callback_loganalytics
        key: include_content
  include_task_args:
    description: Send the task args to the Azure Log Analytics workspace.
    type: bool
    default: false
    env:
      - name: ANSIBLE_LOGANALYTICS_INCLUDE_TASK_ARGS
    ini:
      - section: callback_loganalytics
        key: include_task_args
  stream_name:
    description: The name of the stream used to send the logs to the Azure Log Analytics workspace.
    type: str
    required: true
    env:
      - name: ANSIBLE_LOGANALYTICS_STREAM_NAME
    ini:
      - section: callback_loganalytics
        key: stream_name
  tenant_id:
    description: Tenant ID for the Azure Active Directory.
    type: str
    required: true
    env:
      - name: ANSIBLE_LOGANALYTICS_TENANT_ID
    ini:
      - section: callback_loganalytics
        key: tenant_id
  timeout:
    description: Timeout for the HTTP requests to the Azure Log Analytics API.
    type: int
    default: 2
    env:
      - name: ANSIBLE_LOGANALYTICS_TIMEOUT
    ini:
      - section: callback_loganalytics
        key: timeout
seealso:
  - name: Logs Ingestion API
    description: Overview of Logs Ingestion API in Azure Monitor
    link: https://learn.microsoft.com/en-us/azure/azure-monitor/logs/logs-ingestion-api-overview
notes:
  - Triple verbosity logging (C(-vvv)) can be used to generate JSON sample data for creating the table schema in Azure Log Analytics.
    Search for the string C(Event Data:) in the output in order to locate the data sample.
"""

EXAMPLES = """
examples: |
  Enable the plugin in ansible.cfg:
    [defaults]
    callback_enabled = community.general.loganalytics_ingestion
  Set the environment variables:
    export ANSIBLE_LOGANALYTICS_DCE_URL=https://my-dce.ingest.monitor.azure.com
    export ANSIBLE_LOGANALYTICS_DCR_ID=dcr-xxxxxx
    export ANSIBLE_LOGANALYTICS_CLIENT_ID=xxxxxxxx
    export ANSIBLE_LOGANALYTICS_CLIENT_SECRET=xxxxxxxx
    export ANSIBLE_LOGANALYTICS_TENANT_ID=xxxxxxxx
    export ANSIBLE_LOGANALYTICS_STREAM_NAME=Custom-MyTable
"""

import getpass
import json
import socket
import uuid
from datetime import datetime, timedelta, timezone
from os.path import basename
from urllib.parse import urlencode

from ansible.module_utils.urls import open_url
from ansible.plugins.callback import CallbackBase
from ansible.utils.display import Display

display = Display()


class AzureLogAnalyticsIngestionSource:
    def __init__(
        self,
        dce_url,
        dcr_id,
        disable_attempts,
        disable_on_failure,
        client_id,
        client_secret,
        tenant_id,
        stream_name,
        include_task_args,
        include_content,
        timeout,
        fqcn,
    ):
        self.dce_url = dce_url
        self.dcr_id = dcr_id
        self.disabled = False
        self.disable_attempts = disable_attempts
        self.disable_on_failure = disable_on_failure
        self.client_id = client_id
        self.client_secret = client_secret
        self.failures = 0
        self.tenant_id = tenant_id
        self.stream_name = stream_name
        self.include_task_args = include_task_args
        self.include_content = include_content
        self.token_expiration_time = None
        self.session = str(uuid.uuid4())
        self.host = socket.gethostname()
        self.user = getpass.getuser()
        self.timeout = timeout
        self.fqcn = fqcn

        self.bearer_token = self.get_bearer_token()

    # OAuth2 authentication method to get a Bearer token
    # This replaces the shared_key authentication mechanism
    def get_bearer_token(self):
        url = f"https://login.microsoftonline.com/{self.tenant_id}/oauth2/v2.0/token"
        headers = {"Content-Type": "application/x-www-form-urlencoded"}
        data = urlencode(
            {
                "grant_type": "client_credentials",
                "client_id": self.client_id,
                "client_secret": self.client_secret,
                # The scope value comes from https://learn.microsoft.com/en-us/azure/azure-monitor/logs/logs-ingestion-api-overview#headers
                # and https://learn.microsoft.com/en-us/entra/identity-platform/scopes-oidc#the-default-scope
                "scope": "https://monitor.azure.com/.default",
            }
        )
        response = open_url(url, data=data, force=True, headers=headers, method="POST", timeout=self.timeout)
        j = json.loads(response.read().decode("utf-8"))
        self.token_expiration_time = datetime.now() + timedelta(seconds=j.get("expires_in"))
        return j.get("access_token")

    def is_token_valid(self):
        return datetime.now() + timedelta(seconds=10) < self.token_expiration_time

    # Method to send event data to the Azure Logs Ingestion API
    # This replaces the legacy API call and now uses the Logs Ingestion API endpoint
    def send_event(self, event_data):
        if not self.is_token_valid():
            self.bearer_token = self.get_bearer_token()
        ingestion_url = (
            f"{self.dce_url}/dataCollectionRules/{self.dcr_id}/streams/{self.stream_name}?api-version=2023-01-01"
        )
        headers = {"Authorization": f"Bearer {self.bearer_token}", "Content-Type": "application/json"}
        open_url(ingestion_url, data=json.dumps(event_data), headers=headers, method="POST", timeout=self.timeout)

    def _rfc1123date(self):
        return datetime.now(timezone.utc).strftime("%a, %d %b %Y %H:%M:%S GMT")

    # This method wraps the private method with the appropriate error handling.
    def send_to_loganalytics(self, playbook_name, result, state):
        if self.disabled:
            return
        try:
            self._send_to_loganalytics(playbook_name, result, state)
        except Exception as e:
            display.warning(f"{self.fqcn} callback plugin failure: {e}.")
            if self.disable_on_failure:
                self.failures += 1
                if self.failures >= self.disable_attempts:
                    display.warning(
                        f"{self.fqcn} callback plugin failures exceed maximum of '{self.disable_attempts}'!  Disabling plugin!"
                    )
                    self.disabled = True
                else:
                    display.v(f"{self.fqcn} callback plugin failure {self.failures}/{self.disable_attempts}")

    def _send_to_loganalytics(self, playbook_name, result, state):
        ansible_role = str(result._task._role) if result._task._role else None

        # Include/Exclude task args
        if not self.include_task_args:
            result._task_fields.pop("args", None)

        # Include/Exclude content
        if not self.include_content:
            result._result.pop("content", None)

        # Build the event data
        event_data = [
            {
                "TimeGenerated": self._rfc1123date(),
                "Host": result._host.name,
                "User": self.user,
                "Playbook": playbook_name,
                "Role": ansible_role,
                "TaskName": result._task.get_name(),
                "Task": result._task_fields,
                "Action": result._task_fields["action"],
                "State": state,
                "Result": result._result,
                "Session": self.session,
            }
        ]

        # The data displayed here can be used as a sample file in order to create the table's schema.
        display.vvv(f"Event Data: {json.dumps(event_data)}")

        self.send_event(event_data)


class CallbackModule(CallbackBase):
    CALLBACK_VERSION = 2.0
    CALLBACK_TYPE = "notification"
    CALLBACK_NAME = "loganalytics_ingestion"
    CALLBACK_NEEDS_ENABLED = True

    def __init__(self, display=None):
        super().__init__(display=display)
        self.start_datetimes = {}
        self.playbook_name = None
        self.azure_loganalytics = None
        self.fqcn = f"community.general.{self.CALLBACK_NAME}"

    def set_options(self, task_keys=None, var_options=None, direct=None):
        super().set_options(task_keys=task_keys, var_options=var_options, direct=direct)

        # Set options for the new Azure Logs Ingestion API configuration
        self.client_id = self.get_option("client_id")
        self.client_secret = self.get_option("client_secret")
        self.dce_url = self.get_option("dce_url")
        self.dcr_id = self.get_option("dcr_id")
        self.disable_attempts = self.get_option("disable_attempts")
        self.disable_on_failure = self.get_option("disable_on_failure")
        self.include_content = self.get_option("include_content")
        self.include_task_args = self.get_option("include_task_args")
        self.stream_name = self.get_option("stream_name")
        self.tenant_id = self.get_option("tenant_id")
        self.timeout = self.get_option("timeout")

        # Initialize the AzureLogAnalyticsIngestionSource with the new settings
        self.azure_loganalytics = AzureLogAnalyticsIngestionSource(
            self.dce_url,
            self.dcr_id,
            self.disable_attempts,
            self.disable_on_failure,
            self.client_id,
            self.client_secret,
            self.tenant_id,
            self.stream_name,
            self.include_task_args,
            self.include_content,
            self.timeout,
            self.fqcn,
        )

    def v2_playbook_on_start(self, playbook):
        self.playbook_name = basename(playbook._file_name)

    # Build event data and send it to the Logs Ingestion API
    def v2_runner_on_failed(self, result, **kwargs):
        self.azure_loganalytics.send_to_loganalytics(self.playbook_name, result, "FAILED")

    def v2_runner_on_ok(self, result, **kwargs):
        self.azure_loganalytics.send_to_loganalytics(self.playbook_name, result, "OK")



#
# (c) 2016 Red Hat Inc.
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#
from __future__ import absolute_import, division, print_function


__metaclass__ = type


from ansible.utils.display import Display
from ansible_collections.ansible.netcommon.plugins.action.network import (
    ActionModule as ActionNetworkModule,
)


display = Display()


class ActionModule(ActionNetworkModule):
    def run(self, tmp=None, task_vars=None):
        del tmp  # tmp no longer has any effect

        module_name = self._task.action.split(".")[-1]
        self._config_module = True if module_name in ["ios_config", "config"] else False
        persistent_connection = self._play_context.connection.split(".")[-1]

        if persistent_connection != "network_cli":
            return {
                "failed": True,
                "msg": "Connection type %s is not valid for this module"
                % self._play_context.connection,
            }

        return super(ActionModule, self).run(task_vars=task_vars)



#!/usr/bin/python
#
# Copyright Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: ipmi_power
short_description: Power management for machine
description:
  - Use this module for power management.
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  name:
    description:
      - Hostname or IP address of the BMC.
    required: true
    type: str
  port:
    description:
      - Remote RMCP port.
    default: 623
    type: int
  user:
    description:
      - Username to use to connect to the BMC.
    required: true
    type: str
  password:
    description:
      - Password to connect to the BMC.
    required: true
    type: str
  key:
    description:
      - Encryption key to connect to the BMC in hex format.
    type: str
    version_added: 4.1.0
  state:
    description:
      - Whether to ensure that the machine in desired state.
      - 'The choices for state are:'
      - V(on) -- Request system turn on.
      - V(off) -- Request system turn off without waiting for OS to shutdown.
      - V(shutdown) -- Have system request OS proper shutdown.
      - V(reset) -- Request system reset without waiting for OS.
      - V(boot) -- If system is off, then V(on), else V(reset).
      - Either this option or O(machine) is required.
    choices: ['on', 'off', shutdown, reset, boot]
    type: str
  timeout:
    description:
      - Maximum number of seconds before interrupt request.
    default: 300
    type: int
  machine:
    description:
      - Provide a list of the remote target address for the bridge IPMI request, and the power status.
      - Either this option or O(state) is required.
    type: list
    elements: dict
    version_added: 4.3.0
    suboptions:
      targetAddress:
        description:
          - Remote target address for the bridge IPMI request.
        type: int
        required: true
      state:
        description:
          - Whether to ensure that the machine specified by O(machine[].targetAddress) in desired state.
          - If this option is not set, the power state is set by O(state).
          - If both this option and O(state) are set, this option takes precedence over O(state).
        choices: ['on', 'off', shutdown, reset, boot]
        type: str

requirements:
  - pyghmi
author: "Bulat Gaifullin (@bgaifullin) <gaifullinbf@gmail.com>"
"""

RETURN = r"""
powerstate:
  description: The current power state of the machine.
  returned: success and O(machine) is not provided
  type: str
  sample: 'on'
status:
  description: The current power state of the machine when the machine option is set.
  returned: success and O(machine) is provided
  type: list
  elements: dict
  version_added: 4.3.0
  contains:
    powerstate:
      description: The current power state of the machine specified by RV(status[].targetAddress).
      type: str
    targetAddress:
      description: The remote target address.
      type: int
  sample:
    [
      {
        "powerstate": "on",
        "targetAddress": 48
      },
      {
        "powerstate": "on",
        "targetAddress": 50
      }
    ]
"""

EXAMPLES = r"""
- name: Ensure machine is powered on
  community.general.ipmi_power:
    name: test.testdomain.com
    user: admin
    password: password
    state: 'on'

- name: Ensure machines of which remote target address is 48 and 50 are powered off
  community.general.ipmi_power:
    name: test.testdomain.com
    user: admin
    password: password
    state: 'off'
    machine:
      - targetAddress: 48
      - targetAddress: 50

- name: Ensure machine of which remote target address is 48 is powered on, and 50 is powered off
  community.general.ipmi_power:
    name: test.testdomain.com
    user: admin
    password: password
    machine:
      - targetAddress: 48
        state: 'on'
      - targetAddress: 50
        state: 'off'
"""

import binascii
import traceback

PYGHMI_IMP_ERR = None
INVALID_TARGET_ADDRESS = 0x100
try:
    from pyghmi.ipmi import command
except ImportError:
    PYGHMI_IMP_ERR = traceback.format_exc()
    command = None

from ansible.module_utils.basic import AnsibleModule, missing_required_lib


def main():
    module = AnsibleModule(
        argument_spec=dict(
            name=dict(required=True),
            port=dict(default=623, type="int"),
            state=dict(choices=["on", "off", "shutdown", "reset", "boot"]),
            user=dict(required=True, no_log=True),
            password=dict(required=True, no_log=True),
            key=dict(type="str", no_log=True),
            timeout=dict(default=300, type="int"),
            machine=dict(
                type="list",
                elements="dict",
                options=dict(
                    targetAddress=dict(required=True, type="int"),
                    state=dict(type="str", choices=["on", "off", "shutdown", "reset", "boot"]),
                ),
            ),
        ),
        supports_check_mode=True,
        required_one_of=(["state", "machine"],),
    )

    if command is None:
        module.fail_json(msg=missing_required_lib("pyghmi"), exception=PYGHMI_IMP_ERR)

    name = module.params["name"]
    port = module.params["port"]
    user = module.params["user"]
    password = module.params["password"]
    state = module.params["state"]
    timeout = module.params["timeout"]
    machine = module.params["machine"]

    try:
        if module.params["key"]:
            key = binascii.unhexlify(module.params["key"])
        else:
            key = None
    except Exception:
        module.fail_json(msg="Unable to convert 'key' from hex string.")

    # --- run command ---
    try:
        ipmi_cmd = command.Command(bmc=name, userid=user, password=password, port=port, kg=key)
        module.debug(f'ipmi instantiated - name: "{name}"')

        changed = False
        if machine is None:
            current = ipmi_cmd.get_power()
            if current["powerstate"] != state:
                response = {"powerstate": state} if module.check_mode else ipmi_cmd.set_power(state, wait=timeout)
                changed = True
            else:
                response = current

            if "error" in response:
                module.fail_json(msg=response["error"])

            module.exit_json(changed=changed, **response)
        else:
            response = []
            for entry in machine:
                taddr = entry["targetAddress"]
                if taddr >= INVALID_TARGET_ADDRESS:
                    module.fail_json(msg="targetAddress should be set between 0 to 255.")

                try:
                    # bridge_request is supported on pyghmi 1.5.30 and later
                    current = ipmi_cmd.get_power(bridge_request={"addr": taddr})
                except TypeError:
                    module.fail_json(msg="targetAddress isn't supported on the installed pyghmi.")

                if entry["state"]:
                    tstate = entry["state"]
                elif state:
                    tstate = state
                else:
                    module.fail_json(msg="Either state or suboption of machine state should be set.")

                if current["powerstate"] != tstate:
                    changed = True
                    if not module.check_mode:
                        new = ipmi_cmd.set_power(tstate, wait=timeout, bridge_request={"addr": taddr})
                        if "error" in new:
                            module.fail_json(msg=new["error"])

                        response.append({"targetAddress:": taddr, "powerstate": new["powerstate"]})

                if current["powerstate"] == tstate or module.check_mode:
                    response.append({"targetAddress:": taddr, "powerstate": tstate})

            module.exit_json(changed=changed, status=response)
    except Exception as e:
        module.fail_json(msg=str(e))


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage CheckPoint Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import (absolute_import, division, print_function)

__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = """
---
module: cp_mgmt_if_map_server
short_description: Manages if-map-server objects on Checkpoint over Web Services API
description:
  - Manages if-map-server objects on Checkpoint devices including creating, updating and removing objects.
  - All operations are performed over Web Services API.
  - Available from R82 JHF management version.
version_added: "6.5.0"
author: "Dor Berenstein (@chkp-dorbe)"
options:
  name:
    description:
      - Object name.
    type: str
    required: True
  host:
    description:
      - Host that is IF-MAP server. Identified by name or UID.
    type: str
  monitored_ips:
    description:
      - IP ranges to be monitored by the IF-MAP client.
    type: list
    elements: dict
    suboptions:
      first_ip:
        description:
          - First IPv4 address in the range to be monitored.
        type: str
      last_ip:
        description:
          - Last IPv4 address in the range to be monitored.
        type: str
  port:
    description:
      - IF-MAP server port number.
    type: int
  server_version:
    description:
      - IF-MAP version.
    type: str
    choices: ['2.0', '1.1']
  path:
    description:
      - N/A
    type: str
  query_whole_ranges:
    description:
      - Indicate whether to query whole ranges instead of single IP.
    type: bool
  authentication:
    description:
      - Authentication configuration for the IF-MAP server.
    type: dict
    suboptions:
      authentication_method:
        description:
          - Authentication method for the IF-MAP server.
        type: str
        choices: ['certificate_based', 'basic']
      username:
        description:
          - Username for the IF-MAP server authentication. <font color="red">Required only when</font> 'authentication-method' is set to 'basic'.
        type: str
      password:
        description:
          - Username for the IF-MAP server authentication. <font color="red">Required only when</font> 'authentication-method' is set to 'basic'.
        type: str
  color:
    description:
      - Color of the object. Should be one of existing colors.
    type: str
    choices: ['aquamarine', 'black', 'blue', 'crete blue', 'burlywood', 'cyan', 'dark green', 'khaki', 'orchid', 'dark orange', 'dark sea green',
             'pink', 'turquoise', 'dark blue', 'firebrick', 'brown', 'forest green', 'gold', 'dark gold', 'gray', 'dark gray', 'light green', 'lemon chiffon',
             'coral', 'sea green', 'sky blue', 'magenta', 'purple', 'slate blue', 'violet red', 'navy blue', 'olive', 'orange', 'red', 'sienna', 'yellow']
  comments:
    description:
      - Comments string.
    type: str
  details_level:
    description:
      - The level of detail for some of the fields in the response can vary from showing only the UID value of the object to a fully detailed
        representation of the object.
    type: str
    choices: ['uid', 'standard', 'full']
  domains_to_process:
    description:
      - Indicates which domains to process the commands on. It cannot be used with the details-level full, must be run from the System Domain only and
        with ignore-warnings true. Valid values are, CURRENT_DOMAIN, ALL_DOMAINS_ON_THIS_SERVER.
    type: list
    elements: str
  tags:
    description:
      - Collection of tag identifiers.
    type: list
    elements: str
  ignore_warnings:
    description:
      - Apply changes ignoring warnings.
    type: bool
  ignore_errors:
    description:
      - Apply changes ignoring errors. You won't be able to publish such a changes. If ignore-warnings flag was omitted - warnings will also be ignored.
    type: bool
extends_documentation_fragment: check_point.mgmt.checkpoint_objects
"""

EXAMPLES = """
- name: add-if-map-server
  cp_mgmt_if_map_server:
    authentication:
      authentication_method: certificate_based
    host: TestHost
    monitored_ips:
    - first_ip: 1.1.1.1
      last_ip: 1.1.1.2
    - first_ip: 2.1.1.1
      last_ip: 2.1.1.2
    name: TestIfMapServer
    path: path
    port: 1
    state: present
    version: 2

- name: set-if-map-server
  cp_mgmt_if_map_server:
    host: TestHost2
    monitored_ips:
    - first_ip: 3.1.1.1
      last_ip: 3.1.1.2
    name: TestIfMapServer
    path: newPath
    port: 2
    query_whole_ranges: false
    state: present
    version: 1.1

- name: delete-if-map-server
  cp_mgmt_if_map_server:
    name: TestIdp
    state: absent
"""

RETURN = """
cp_mgmt_if_map_server:
  description: The checkpoint object created or updated.
  returned: always, except when deleting the object.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import checkpoint_argument_spec_for_objects, api_call


def main():
    argument_spec = dict(
        name=dict(type='str', required=True),
        host=dict(type='str'),
        monitored_ips=dict(type='list', elements="dict", options=dict(
            first_ip=dict(type='str'),
            last_ip=dict(type='str')
        )),
        port=dict(type='int'),
        server_version=dict(type='str', choices=['2.0', '1.1']),
        path=dict(type='str'),
        query_whole_ranges=dict(type='bool'),
        authentication=dict(type='dict', options=dict(
            authentication_method=dict(type='str', choices=['certificate_based', 'basic']),
            username=dict(type='str'),
            password=dict(type='str', no_log=True)
        )),
        color=dict(type='str', choices=['aquamarine', 'black', 'blue', 'crete blue', 'burlywood', 'cyan', 'dark green',
                                        'khaki', 'orchid', 'dark orange', 'dark sea green', 'pink', 'turquoise', 'dark blue', 'firebrick', 'brown',
                                        'forest green', 'gold', 'dark gold', 'gray', 'dark gray', 'light green', 'lemon chiffon', 'coral', 'sea green',
                                        'sky blue', 'magenta', 'purple', 'slate blue', 'violet red', 'navy blue', 'olive', 'orange', 'red', 'sienna',
                                        'yellow']),
        comments=dict(type='str'),
        details_level=dict(type='str', choices=['uid', 'standard', 'full']),
        domains_to_process=dict(type='list', elements="str"),
        tags=dict(type='list', elements="str"),
        ignore_warnings=dict(type='bool'),
        ignore_errors=dict(type='bool')
    )
    argument_spec.update(checkpoint_argument_spec_for_objects)

    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=True)
    api_call_object = 'if-map-server'

    result = api_call(module, api_call_object)
    module.exit_json(**result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: networks_clients_provision
short_description: Resource module for networks _clients _provision
description:
  - Manage operation create of the resource networks _clients _provision.
  - Provisions a client with a name and policy. Clients can be provisioned before
    they associate to the network.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  clients:
    description: The array of clients to provision.
    elements: dict
    suboptions:
      mac:
        description: The MAC address of the client. Required.
        type: str
      name:
        description: The display name for the client. Optional. Limited to 255 bytes.
        type: str
    type: list
  devicePolicy:
    description: The policy to apply to the specified client. Can be 'Group policy',
      'Allowed', 'Blocked', 'Per connection' or 'Normal'. Required.
    type: str
  groupPolicyId:
    description: The ID of the desired group policy to apply to the client. Required
      if 'devicePolicy' is set to "Group policy". Otherwise this is ignored.
    type: str
  networkId:
    description: NetworkId path parameter. Network ID.
    type: str
  policiesBySecurityAppliance:
    description: An object, describing what the policy-connection association is for
      the security appliance. (Only relevant if the security appliance is actually
      within the network).
    suboptions:
      devicePolicy:
        description: The policy to apply to the specified client. Can be 'Allowed',
          'Blocked' or 'Normal'. Required.
        type: str
    type: dict
  policiesBySsid:
    description: An object, describing the policy-connection associations for each
      active SSID within the network. Keys should be the number of enabled SSIDs,
      mapping to an object describing the client's policy.
    suboptions:
      '0':
        description: The number for the SSID.
        suboptions:
          devicePolicy:
            description: The policy to apply to the specified client. Can be 'Allowed',
              'Blocked', 'Normal' or 'Group policy'. Required.
            type: str
          groupPolicyId:
            description: The ID of the desired group policy to apply to the client.
              Required if 'devicePolicy' is set to "Group policy". Otherwise this
              is ignored.
            type: str
        type: dict
      '1':
        description: The number for the SSID.
        suboptions:
          devicePolicy:
            description: The policy to apply to the specified client. Can be 'Allowed',
              'Blocked', 'Normal' or 'Group policy'. Required.
            type: str
          groupPolicyId:
            description: The ID of the desired group policy to apply to the client.
              Required if 'devicePolicy' is set to "Group policy". Otherwise this
              is ignored.
            type: str
        type: dict
      '10':
        description: The number for the SSID.
        suboptions:
          devicePolicy:
            description: The policy to apply to the specified client. Can be 'Allowed',
              'Blocked', 'Normal' or 'Group policy'. Required.
            type: str
          groupPolicyId:
            description: The ID of the desired group policy to apply to the client.
              Required if 'devicePolicy' is set to "Group policy". Otherwise this
              is ignored.
            type: str
        type: dict
      '11':
        description: The number for the SSID.
        suboptions:
          devicePolicy:
            description: The policy to apply to the specified client. Can be 'Allowed',
              'Blocked', 'Normal' or 'Group policy'. Required.
            type: str
          groupPolicyId:
            description: The ID of the desired group policy to apply to the client.
              Required if 'devicePolicy' is set to "Group policy". Otherwise this
              is ignored.
            type: str
        type: dict
      '12':
        description: The number for the SSID.
        suboptions:
          devicePolicy:
            description: The policy to apply to the specified client. Can be 'Allowed',
              'Blocked', 'Normal' or 'Group policy'. Required.
            type: str
          groupPolicyId:
            description: The ID of the desired group policy to apply to the client.
              Required if 'devicePolicy' is set to "Group policy". Otherwise this
              is ignored.
            type: str
        type: dict
      '13':
        description: The number for the SSID.
        suboptions:
          devicePolicy:
            description: The policy to apply to the specified client. Can be 'Allowed',
              'Blocked', 'Normal' or 'Group policy'. Required.
            type: str
          groupPolicyId:
            description: The ID of the desired group policy to apply to the client.
              Required if 'devicePolicy' is set to "Group policy". Otherwise this
              is ignored.
            type: str
        type: dict
      '14':
        description: The number for the SSID.
        suboptions:
          devicePolicy:
            description: The policy to apply to the specified client. Can be 'Allowed',
              'Blocked', 'Normal' or 'Group policy'. Required.
            type: str
          groupPolicyId:
            description: The ID of the desired group policy to apply to the client.
              Required if 'devicePolicy' is set to "Group policy". Otherwise this
              is ignored.
            type: str
        type: dict
      '2':
        description: The number for the SSID.
        suboptions:
          devicePolicy:
            description: The policy to apply to the specified client. Can be 'Allowed',
              'Blocked', 'Normal' or 'Group policy'. Required.
            type: str
          groupPolicyId:
            description: The ID of the desired group policy to apply to the client.
              Required if 'devicePolicy' is set to "Group policy". Otherwise this
              is ignored.
            type: str
        type: dict
      '3':
        description: The number for the SSID.
        suboptions:
          devicePolicy:
            description: The policy to apply to the specified client. Can be 'Allowed',
              'Blocked', 'Normal' or 'Group policy'. Required.
            type: str
          groupPolicyId:
            description: The ID of the desired group policy to apply to the client.
              Required if 'devicePolicy' is set to "Group policy". Otherwise this
              is ignored.
            type: str
        type: dict
      '4':
        description: The number for the SSID.
        suboptions:
          devicePolicy:
            description: The policy to apply to the specified client. Can be 'Allowed',
              'Blocked', 'Normal' or 'Group policy'. Required.
            type: str
          groupPolicyId:
            description: The ID of the desired group policy to apply to the client.
              Required if 'devicePolicy' is set to "Group policy". Otherwise this
              is ignored.
            type: str
        type: dict
      '5':
        description: The number for the SSID.
        suboptions:
          devicePolicy:
            description: The policy to apply to the specified client. Can be 'Allowed',
              'Blocked', 'Normal' or 'Group policy'. Required.
            type: str
          groupPolicyId:
            description: The ID of the desired group policy to apply to the client.
              Required if 'devicePolicy' is set to "Group policy". Otherwise this
              is ignored.
            type: str
        type: dict
      '6':
        description: The number for the SSID.
        suboptions:
          devicePolicy:
            description: The policy to apply to the specified client. Can be 'Allowed',
              'Blocked', 'Normal' or 'Group policy'. Required.
            type: str
          groupPolicyId:
            description: The ID of the desired group policy to apply to the client.
              Required if 'devicePolicy' is set to "Group policy". Otherwise this
              is ignored.
            type: str
        type: dict
      '7':
        description: The number for the SSID.
        suboptions:
          devicePolicy:
            description: The policy to apply to the specified client. Can be 'Allowed',
              'Blocked', 'Normal' or 'Group policy'. Required.
            type: str
          groupPolicyId:
            description: The ID of the desired group policy to apply to the client.
              Required if 'devicePolicy' is set to "Group policy". Otherwise this
              is ignored.
            type: str
        type: dict
      '8':
        description: The number for the SSID.
        suboptions:
          devicePolicy:
            description: The policy to apply to the specified client. Can be 'Allowed',
              'Blocked', 'Normal' or 'Group policy'. Required.
            type: str
          groupPolicyId:
            description: The ID of the desired group policy to apply to the client.
              Required if 'devicePolicy' is set to "Group policy". Otherwise this
              is ignored.
            type: str
        type: dict
      '9':
        description: The number for the SSID.
        suboptions:
          devicePolicy:
            description: The policy to apply to the specified client. Can be 'Allowed',
              'Blocked', 'Normal' or 'Group policy'. Required.
            type: str
          groupPolicyId:
            description: The ID of the desired group policy to apply to the client.
              Required if 'devicePolicy' is set to "Group policy". Otherwise this
              is ignored.
            type: str
        type: dict
    type: dict
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for networks provisionNetworkClients
    description: Complete reference of the provisionNetworkClients API.
    link: https://developer.cisco.com/meraki/api-v1/#!provision-network-clients
notes:
  - SDK Method used are
    networks.Networks.provision_network_clients,
  - Paths used are
    post /networks/{networkId}/clients/provision,
"""

EXAMPLES = r"""
- name: Create
  cisco.meraki.networks_clients_provision:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    clients:
      - mac: 00:11:22:33:44:55
        name: Miles's phone
    devicePolicy: Group policy
    groupPolicyId: '101'
    networkId: string
    policiesBySecurityAppliance:
      devicePolicy: Normal
    policiesBySsid:
      '0':
        devicePolicy: Group policy
        groupPolicyId: '101'
      '1':
        devicePolicy: Group policy
        groupPolicyId: '101'
      '10':
        devicePolicy: Group policy
        groupPolicyId: '101'
      '11':
        devicePolicy: Group policy
        groupPolicyId: '101'
      '12':
        devicePolicy: Group policy
        groupPolicyId: '101'
      '13':
        devicePolicy: Group policy
        groupPolicyId: '101'
      '14':
        devicePolicy: Group policy
        groupPolicyId: '101'
      '2':
        devicePolicy: Group policy
        groupPolicyId: '101'
      '3':
        devicePolicy: Group policy
        groupPolicyId: '101'
      '4':
        devicePolicy: Group policy
        groupPolicyId: '101'
      '5':
        devicePolicy: Group policy
        groupPolicyId: '101'
      '6':
        devicePolicy: Group policy
        groupPolicyId: '101'
      '7':
        devicePolicy: Group policy
        groupPolicyId: '101'
      '8':
        devicePolicy: Group policy
        groupPolicyId: '101'
      '9':
        devicePolicy: Group policy
        groupPolicyId: '101'
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "clients": [
        {
          "mac": "string",
          "clientId": "string",
          "name": "string",
          "message": "string"
        }
      ],
      "devicePolicy": "string",
      "groupPolicyId": "string"
    }
"""



#!/usr/bin/python

# Copyright (c) 2026, Dexter Le <dextersydney2001@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: kopia_repository
short_description: Manage Kopia repository
author:
  - Dexter Le (@munchtoast)
version_added: "13.1.0"
description:
  - Manage a Kopia repository using the Kopia CLI.
  - Supports creating, connecting, disconnecting, syncing, and throttling repositories.
extends_documentation_fragment:
  - community.general._attributes
  - community.general._kopia
attributes:
  check_mode:
    support: full
  diff_mode:
    support: full
options:
  state:
    description:
      - Desired state of the Kopia repository.
    type: str
    choices:
      created: Creates a new repository at the given backend.
      connected: Connects to an existing repository or Kopia server.
      disconnected: Disconnects from the current repository.
      synced: Synchronizes the current repository to another backend location.
      throttled: Sets throttle limits on the current repository.
    default: created
  fingerprint_tls:
    description:
      - TLS certificate fingerprint of the Kopia server.
      - Required if O(state=connected) and O(backend.provider=server).
    type: str
  url:
    description:
      - URL of the Kopia server to connect to.
      - Required if O(state=connected) and O(backend.provider=server).
    type: str
  throttle:
    description:
      - Throttle limits for the repository connection.
      - Only used when O(state=throttled).
    type: dict
    suboptions:
      download_bytes_per_second:
        description:
          - Maximum download speed in bytes per second. Set to V(0) to disable the limit.
        type: int
      upload_bytes_per_second:
        description:
          - Maximum upload speed in bytes per second. Set to V(0) to disable the limit.
        type: int
      read_requests_per_second:
        description:
          - Maximum number of read requests per second.
        type: float
      write_requests_per_second:
        description:
          - Maximum number of write requests per second.
        type: float
      list_requests_per_second:
        description:
          - Maximum number of list requests per second.
        type: float
      concurrent_reads:
        description:
          - Maximum number of concurrent read operations.
        type: int
      concurrent_writes:
        description:
          - Maximum number of concurrent write operations.
        type: int
  backend:
    description:
      - Backend storage configuration for the repository.
      - Required if O(state=created), O(state=connected), or O(state=synced).
    type: dict
    suboptions:
      provider:
        description:
          - Backend storage provider.
          - Use V(server) to connect to a Kopia repository server instead of directly to storage.
        type: str
        required: true
        choices: [azure, b2, filesystem, gcs, gdrive, rclone, s3, sftp, webdav, server]
      bucket:
        description:
          - Bucket name for the backend.
          - Required if O(backend.provider=b2), O(backend.provider=gcs), or O(backend.provider=s3).
        type: str
      container:
        description:
          - Azure Blob Storage container name.
          - Required if O(backend.provider=azure).
        type: str
      storage_account:
        description:
          - Azure storage account name.
          - Required if O(backend.provider=azure).
        type: str
      storage_key:
        description:
          - Azure storage account key used to authenticate.
          - Optional if O(backend.provider=azure); omit when using managed identity or SAS tokens.
        type: str
      sas_token:
        description:
          - Azure Shared Access Signature token for authentication.
          - Optional alternative to O(backend.storage_key) when O(backend.provider=azure).
        type: str
      storage_domain:
        description:
          - Azure storage domain override.
          - Optional if O(backend.provider=azure).
        type: str
      access_key:
        description:
          - Access key ID for the backend.
          - Required if O(backend.provider=b2) or O(backend.provider=s3).
        type: str
      secret_access_key:
        description:
          - Secret access key for the backend.
          - Required if O(backend.provider=b2) or O(backend.provider=s3).
        type: str
      session_token:
        description:
          - Session token for temporary AWS credentials.
          - Optional if O(backend.provider=s3).
        type: str
      endpoint:
        description:
          - S3-compatible endpoint URL.
          - Optional if O(backend.provider=s3); defaults to C(s3.amazonaws.com).
        type: str
      region:
        description:
          - S3 bucket region.
          - Optional if O(backend.provider=s3).
        type: str
      folder_id:
        description:
          - Google Drive folder ID to use as the backend root.
          - Required if O(backend.provider=gdrive).
        type: str
      credentials_file:
        description:
          - Path to a JSON credentials file for authentication.
          - Optional if O(backend.provider=gcs) or O(backend.provider=gdrive).
        type: path
      path:
        description:
          - Local file system path or remote path for the backend.
          - Required if O(backend.provider=filesystem), O(backend.provider=rclone), or O(backend.provider=sftp).
        type: path
      host:
        description:
          - SFTP server hostname.
          - Required if O(backend.provider=sftp).
        type: str
      username:
        description:
          - SFTP username for authentication.
          - Required if O(backend.provider=sftp).
        type: str
      port:
        description:
          - SFTP server port.
          - Optional if O(backend.provider=sftp); defaults to V(22).
        type: int
      keyfile:
        description:
          - Path to the SSH private key file for SFTP authentication.
          - Optional if O(backend.provider=sftp).
        type: path
      known_hosts:
        description:
          - Path to a known_hosts file for SFTP host key verification.
          - Optional if O(backend.provider=sftp).
        type: path
      url:
        description:
          - WebDAV server URL.
          - Required if O(backend.provider=webdav).
        type: str
      webdav_username:
        description:
          - Username for WebDAV authentication.
          - Optional if O(backend.provider=webdav).
        type: str
      webdav_password:
        description:
          - Password for WebDAV authentication.
          - Optional if O(backend.provider=webdav).
        type: str
      prefix:
        description:
          - Object key prefix within the backend storage.
          - Optional if O(backend.provider=azure), O(backend.provider=b2),
            O(backend.provider=gcs), or O(backend.provider=s3).
        type: str
"""

EXAMPLES = r"""
- name: Create a Kopia repository with S3 backend
  community.general.kopia_repository:
    state: created
    password: secret
    config: /etc/kopia/root.config
    backend:
      provider: s3
      bucket: my-bucket
      access_key: myaccesskey
      secret_access_key: mysecretaccesskey

- name: Create a Kopia repository on the local filesystem
  community.general.kopia_repository:
    state: created
    password: secret
    backend:
      provider: filesystem
      path: /mnt/backup/kopia

- name: Connect to a Kopia repository server
  community.general.kopia_repository:
    state: connected
    password: secret
    config: /etc/kopia/root.config
    url: https://kopia.example.com:51515
    fingerprint_tls: AA:BB:CC:DD:EE:FF
    backend:
      provider: server

- name: Connect directly to an Azure backend
  community.general.kopia_repository:
    state: connected
    password: secret
    backend:
      provider: azure
      container: my-container
      storage_account: mystorageaccount
      storage_key: mystoragekey

- name: Disconnect the Kopia repository
  community.general.kopia_repository:
    state: disconnected
    config: /etc/kopia/root.config

- name: Sync Kopia repository to an S3 location
  community.general.kopia_repository:
    state: synced
    password: secret
    config: /etc/kopia/root.config
    backend:
      provider: s3
      bucket: my-synced-bucket
      access_key: myaccesskey
      secret_access_key: mysecretaccesskey
"""

RETURN = r"""
kopia_repository:
  description: Output from the Kopia repository command.
  type: str
  sample: |-
    Connected to repository: s3:/my-bucket/
    Config file: /etc/kopia/root.config
    ...
  returned: always
"""

from ansible_collections.community.general.plugins.module_utils._kopia import (
    KOPIA_COMMON_ARGUMENT_SPEC,
    kopia_runner,
)
from ansible_collections.community.general.plugins.module_utils._module_helper import StateModuleHelper


class KopiaRepository(StateModuleHelper):
    module = dict(
        supports_check_mode=True,
        argument_spec=dict(
            **KOPIA_COMMON_ARGUMENT_SPEC,
            state=dict(
                type="str",
                default="created",
                choices=["created", "connected", "disconnected", "synced", "throttled"],
            ),
            fingerprint_tls=dict(type="str"),
            url=dict(type="str"),
            throttle=dict(
                type="dict",
                options=dict(
                    download_bytes_per_second=dict(type="int"),
                    upload_bytes_per_second=dict(type="int"),
                    read_requests_per_second=dict(type="float"),
                    write_requests_per_second=dict(type="float"),
                    list_requests_per_second=dict(type="float"),
                    concurrent_reads=dict(type="int"),
                    concurrent_writes=dict(type="int"),
                ),
            ),
            backend=dict(
                type="dict",
                options=dict(
                    provider=dict(
                        type="str",
                        required=True,
                        choices=[
                            "azure",
                            "b2",
                            "filesystem",
                            "gcs",
                            "gdrive",
                            "rclone",
                            "s3",
                            "sftp",
                            "webdav",
                            "server",
                        ],
                    ),
                    bucket=dict(type="str"),
                    container=dict(type="str"),
                    storage_account=dict(type="str"),
                    storage_key=dict(type="str", no_log=True),
                    sas_token=dict(type="str", no_log=True),
                    storage_domain=dict(type="str"),
                    access_key=dict(type="str", no_log=True),
                    secret_access_key=dict(type="str", no_log=True),
                    session_token=dict(type="str", no_log=True),
                    endpoint=dict(type="str"),
                    region=dict(type="str"),
                    folder_id=dict(type="str"),
                    credentials_file=dict(type="path"),
                    path=dict(type="path"),
                    host=dict(type="str"),
                    username=dict(type="str"),
                    port=dict(type="int"),
                    keyfile=dict(type="path"),
                    known_hosts=dict(type="path"),
                    url=dict(type="str"),
                    webdav_username=dict(type="str"),
                    webdav_password=dict(type="str", no_log=True),
                    prefix=dict(type="str"),
                ),
                required_if=[
                    ("provider", "azure", ["container", "storage_account"]),
                    ("provider", "b2", ["bucket", "access_key", "secret_access_key"]),
                    ("provider", "filesystem", ["path"]),
                    ("provider", "gcs", ["bucket"]),
                    ("provider", "gdrive", ["folder_id"]),
                    ("provider", "rclone", ["path"]),
                    ("provider", "s3", ["bucket", "access_key", "secret_access_key"]),
                    ("provider", "sftp", ["path", "host", "username"]),
                    ("provider", "webdav", ["url"]),
                ],
            ),
        ),
        required_if=[
            ("state", "created", ["backend"]),
            ("state", "connected", ["backend"]),
            ("state", "synced", ["backend"]),
        ],
    )

    def __init_module__(self):
        self.runner = kopia_runner(self.module)
        self.vars.set("previous_value", self._get()["out"])
        self.vars.set("value", self.vars.previous_value, change=True, diff=True)

    def __quit_module__(self):
        if self.module.check_mode:
            self.vars.set("value", self._predict_value())
        else:
            self.vars.set("value", self._get()["out"])

    def _predict_value(self):
        """Predict the post-operation repository status for check mode change detection."""
        state = self.module.params["state"]
        previous = self.vars.previous_value
        if state in ("created", "connected"):
            return previous if previous is not None else "Connected to repository."
        if state == "disconnected":
            return None if previous is not None else previous
        return previous

    def _get(self):
        with self.runner("status config") as ctx:
            result = ctx.run()
            return dict(
                rc=result[0],
                out=(result[1].rstrip() if result[1] else None),
                err=result[2],
            )

    def _process_command_output(self, fail_on_err, ignore_err_msg=""):
        def process(rc, out, err):
            if fail_on_err and rc != 0 and err and ignore_err_msg not in err:
                self.do_raise(f"kopia failed with error (rc={rc}): {err}")
            out = out.rstrip()
            return None if out == "" else out

        return process

    def state_created(self):
        with self.runner(
            "cli_action state backend password config",
            output_process=self._process_command_output(True, "already exists"),
            check_mode_skip=True,
        ) as ctx:
            ctx.run(cli_action="repository")

    def state_connected(self):
        with self.runner(
            "cli_action state backend password fingerprint_tls url config",
            output_process=self._process_command_output(True, "already connected"),
            check_mode_skip=True,
        ) as ctx:
            ctx.run(cli_action="repository")

    def state_disconnected(self):
        with self.runner(
            "cli_action state password config",
            output_process=self._process_command_output(True, "does not exist"),
            check_mode_skip=True,
        ) as ctx:
            ctx.run(cli_action="repository")

    def state_synced(self):
        with self.runner(
            "cli_action state backend password config",
            output_process=self._process_command_output(True, "already synced"),
            check_mode_skip=True,
        ) as ctx:
            ctx.run(cli_action="repository")

    def state_throttled(self):
        with self.runner(
            "cli_action state throttle_operation throttle config",
            output_process=self._process_command_output(True),
            check_mode_skip=True,
        ) as ctx:
            ctx.run(cli_action="repository", throttle_operation="set")


def main():
    KopiaRepository.execute()


if __name__ == "__main__":
    main()



#
# -*- coding: utf-8 -*-
# Copyright 2020 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The nxos_ospfv3 config file.
It is in this file where the current configuration (as dict)
is compared to the provided configuration (as dict) and the command set
necessary to bring the current configuration to its desired end-state is
created.
"""

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.resource_module import (
    ResourceModule,
)
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import (
    dict_merge,
    get_from_dict,
)

from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.facts.facts import Facts
from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.rm_templates.ospfv3 import (
    Ospfv3Template,
)


class Ospfv3(ResourceModule):
    """
    The nxos_ospfv3 config class
    """

    def __init__(self, module):
        super(Ospfv3, self).__init__(
            empty_fact_val={},
            facts_module=Facts(module),
            module=module,
            resource="ospfv3",
            tmplt=Ospfv3Template(),
        )
        self.parsers = [
            "auto_cost",
            "flush_routes",
            "graceful_restart.set",
            "graceful_restart.helper_disable",
            "graceful_restart.grace_period",
            "graceful_restart.planned_only",
            "isolate",
            "log_adjacency_changes",
            "max_lsa",
            "max_metric",
            "name_lookup",
            "passive_interface.default",
            "router_id",
            "shutdown",
            "timers.lsa_arrival",
            "timers.lsa_group_pacing",
            "timers.throttle.lsa",
        ]

    def execute_module(self):
        """Execute the module

        :rtype: A dictionary
        :returns: The result from module execution
        """
        if self.state not in ["parsed", "gathered"]:
            self.generate_commands()
            self.run_commands()
        return self.result

    def generate_commands(self):
        """Generate configuration commands to send based on
        want, have and desired state.
        """
        wantd = {(entry["process_id"]): entry for entry in self.want.get("processes", [])}
        haved = {(entry["process_id"]): entry for entry in self.have.get("processes", [])}

        # turn all lists of dicts into dicts prior to merge
        for entry in wantd, haved:
            self._ospfv3_list_to_dict(entry)

        # if state is merged, merge want onto have and then compare
        if self.state == "merged":
            wantd = dict_merge(haved, wantd)

        # if state is deleted, empty out wantd and set haved to wantd
        if self.state == "deleted":
            haved = {k: v for k, v in haved.items() if k in wantd or not wantd}
            wantd = {}

        # if state is overridden, first remove processes that are in have but not in want
        if self.state in ["overridden", "deleted"]:
            for k, have in haved.items():
                if k not in wantd:
                    self.addcmd(have, "process_id", True)

        for k, want in wantd.items():
            self._compare(want=want, have=haved.pop(k, {}))

    def _compare(self, want, have):
        """Leverages the base class `compare()` method and
        populates the list of commands to be run by comparing
        the `want` and `have` data with the `parsers` defined
        for the Ospfv3 network resource.
        """
        begin = len(self.commands)
        self.compare(parsers=self.parsers, want=want, have=have)
        self._areas_compare(want=want, have=have)
        self._vrfs_compare(want=want, have=have)
        self._af_compare(want=want, have=have)

        if len(self.commands) != begin or (not have and want):
            self.commands.insert(
                begin,
                self._tmplt.render(
                    want or have,
                    "vrf" if "vrf" in (want.keys() or have.keys()) else "process_id",
                    False,
                ),
            )

    def _areas_compare(self, want, have):
        wareas = want.get("areas", {})
        hareas = have.get("areas", {})
        for name, entry in wareas.items():
            self._area_compare(want=entry, have=hareas.pop(name, {}))
        for name, entry in hareas.items():
            self._area_compare(want={}, have=entry)

    def _area_compare(self, want, have):
        parsers = ["area.nssa", "area.nssa.translate", "area.stub"]
        self.compare(parsers=parsers, want=want, have=have)

    def _vrfs_compare(self, want, have):
        wvrfs = want.get("vrfs", {})
        hvrfs = have.get("vrfs", {})
        for name, entry in wvrfs.items():
            self._compare(want=entry, have=hvrfs.pop(name, {}))
        # remove remaining items in have for replaced
        for name, entry in hvrfs.items():
            self.addcmd(entry, "vrf", True)

    def _af_compare(self, want, have):
        parsers = [
            "default_information.originate",
            "distance",
            "maximum_paths",
            "table_map",
            "timers.throttle.spf",
        ]
        waf = want.get("address_family", {})
        haf = have.get("address_family", {})

        cmd_ptr = len(self.commands)

        self._af_areas_compare(want=waf, have=haf)
        self._af_compare_lists(want=waf, have=haf)
        self.compare(parsers=parsers, want=waf, have=haf)

        cmd_ptr_nxt = len(self.commands)
        if cmd_ptr < cmd_ptr_nxt:
            self.commands.insert(cmd_ptr, "address-family ipv6 unicast")

    def _af_areas_compare(self, want, have):
        wareas = want.get("areas", {})
        hareas = have.get("areas", {})
        for name, entry in wareas.items():
            self._af_area_compare(want=entry, have=hareas.pop(name, {}))
        for name, entry in hareas.items():
            self._af_area_compare(want={}, have=entry)

    def _af_area_compare(self, want, have):
        self.compare(parsers=["area.default_cost"], want=want, have=have)
        self._af_area_compare_lists(want=want, have=have)

    def _af_area_compare_lists(self, want, have):
        for attrib in ["filter_list", "ranges"]:
            wdict = want.get(attrib, {})
            hdict = have.get(attrib, {})
            for key, entry in wdict.items():
                if entry != hdict.pop(key, {}):
                    entry["area_id"] = want["area_id"]
                    self.addcmd(entry, "area.{0}".format(attrib), False)
            # remove remaining items in have for replaced
            for entry in hdict.values():
                entry["area_id"] = have["area_id"]
                self.addcmd(entry, "area.{0}".format(attrib), True)

    def _af_compare_lists(self, want, have):
        for attrib in ["summary_address", "redistribute"]:
            wdict = get_from_dict(want, attrib) or {}
            hdict = get_from_dict(have, attrib) or {}

            for key, entry in wdict.items():
                if entry != hdict.pop(key, {}):
                    self.addcmd(entry, attrib, False)
            # remove remaining items in have for replaced
            for entry in hdict.values():
                self.addcmd(entry, attrib, True)

    def _ospfv3_list_to_dict(self, entry):
        for _pid, proc in entry.items():
            proc["areas"] = {entry["area_id"]: entry for entry in proc.get("areas", [])}
            af = proc.get("address_family")
            if af:
                for area in af.get("areas", []):
                    area["ranges"] = {entry["prefix"]: entry for entry in area.get("ranges", [])}
                    area["filter_list"] = {
                        entry["direction"]: entry for entry in area.get("filter_list", [])
                    }
                af["areas"] = {entry["area_id"]: entry for entry in af.get("areas", [])}
                af["summary_address"] = {
                    entry["prefix"]: entry for entry in af.get("summary_address", [])
                }
                af["redistribute"] = {
                    (entry.get("id"), entry["protocol"]): entry
                    for entry in af.get("redistribute", [])
                }
            if "vrfs" in proc:
                proc["vrfs"] = {entry["vrf"]: entry for entry in proc.get("vrfs", [])}
                self._ospfv3_list_to_dict(proc["vrfs"])



#!/usr/bin/python
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#
from __future__ import absolute_import, division, print_function


__metaclass__ = type


DOCUMENTATION = """
module: nxos_igmp_interface
extends_documentation_fragment:
- cisco.nxos.nxos
short_description: Manages IGMP interface configuration.
description:
- Manages IGMP interface configuration settings.
version_added: 1.0.0
author:
- Jason Edelman (@jedelman8)
- Gabriele Gerbino (@GGabriele)
notes:
- Tested against NXOSv 7.3.(0)D1(1) on VIRL
- Unsupported for Cisco MDS
- When C(state=default), supported params will be reset to a default state. These
  include C(version), C(startup_query_interval), C(startup_query_count), C(robustness),
  C(querier_timeout), C(query_mrt), C(query_interval), C(last_member_qrt), C(last_member_query_count),
  C(group_timeout), C(report_llg), and C(immediate_leave).
- When C(state=absent), all configs for C(oif_ps), and C(oif_routemap) will be removed.
- PIM must be enabled to use this module.
- This module is for Layer 3 interfaces.
- Route-map check not performed (same as CLI) check when configuring route-map with
  'static-oif'
- If restart is set to true with other params set, the restart will happen last, i.e.
  after the configuration takes place. However, 'restart' itself is not idempotent
  as it is an action and not configuration.
options:
  interface:
    description:
    - The full interface name for IGMP configuration. e.g. I(Ethernet1/2).
    required: true
    type: str
  version:
    description:
    - IGMP version. It can be 2 or 3 or keyword 'default'.
    choices:
    - '2'
    - '3'
    - default
    type: str
  startup_query_interval:
    description:
    - Query interval used when the IGMP process starts up. The range is from 1 to
      18000 or keyword 'default'. The default is 31.
    type: str
  startup_query_count:
    description:
    - Query count used when the IGMP process starts up. The range is from 1 to 10
      or keyword 'default'. The default is 2.
    type: str
  robustness:
    description:
    - Sets the robustness variable. Values can range from 1 to 7 or keyword 'default'.
      The default is 2.
    type: str
  querier_timeout:
    description:
    - Sets the querier timeout that the software uses when deciding to take over as
      the querier. Values can range from 1 to 65535 seconds or keyword 'default'.
      The default is 255 seconds.
    type: str
  query_mrt:
    description:
    - Sets the response time advertised in IGMP queries. Values can range from 1 to
      25 seconds or keyword 'default'. The default is 10 seconds.
    type: str
  query_interval:
    description:
    - Sets the frequency at which the software sends IGMP host query messages. Values
      can range from 1 to 18000 seconds or keyword 'default'. The default is 125 seconds.
    type: str
  last_member_qrt:
    description:
    - Sets the query interval waited after sending membership reports before the software
      deletes the group state. Values can range from 1 to 25 seconds or keyword 'default'.
      The default is 1 second.
    type: str
  last_member_query_count:
    description:
    - Sets the number of times that the software sends an IGMP query in response to
      a host leave message. Values can range from 1 to 5 or keyword 'default'. The
      default is 2.
    type: str
  group_timeout:
    description:
    - Sets the group membership timeout for IGMPv2. Values can range from 3 to 65,535
      seconds or keyword 'default'. The default is 260 seconds.
    type: str
  report_llg:
    description:
    - Configures report-link-local-groups. Enables sending reports for groups in 224.0.0.0/24.
      Reports are always sent for nonlink local groups. By default, reports are not
      sent for link local groups.
    type: bool
  immediate_leave:
    description:
    - Enables the device to remove the group entry from the multicast routing table
      immediately upon receiving a leave message for the group. Use this command to
      minimize the leave latency of IGMPv2 group memberships on a given IGMP interface
      because the device does not send group-specific queries. The default is disabled.
    type: bool
  oif_routemap:
    description:
    - Configure a routemap for static outgoing interface (OIF) or keyword 'default'.
    type: str
  oif_ps:
    description:
    - Configure prefixes and sources for static outgoing interface (OIF). This is
      a list of dict where each dict has source and prefix defined or just prefix
      if source is not needed. The specified values will be configured on the device
      and if any previous prefix/sources exist, they will be removed. Keyword 'default'
      is also accepted which removes all existing prefix/sources.
    type: raw
  restart:
    description:
    - Restart IGMP. This is NOT idempotent as this is action only.
    type: bool
    default: false
  state:
    description:
    - Manages desired state of the resource.
    default: present
    choices:
    - present
    - absent
    - default
    type: str
"""
EXAMPLES = """
- cisco.nxos.nxos_igmp_interface:
    interface: ethernet1/32
    startup_query_interval: 30
    oif_ps:
      - {prefix: 238.2.2.6}
      - {source: 192.168.0.1, prefix: 238.2.2.5}
    state: present
"""
RETURN = """
proposed:
    description: k/v pairs of parameters passed into module
    returned: always
    type: dict
    sample: {"startup_query_count": "30",
             "oif_ps": [{'prefix': '238.2.2.6'}, {'source': '192.168.0.1', 'prefix': '238.2.2.5'}]}
existing:
    description: k/v pairs of existing igmp_interface configuration
    returned: always
    type: dict
    sample: {"startup_query_count": "2", "oif_ps": []}
end_state:
    description: k/v pairs of igmp interface configuration after module execution
    returned: always
    type: dict
    sample: {"startup_query_count": "30",
             "oif_ps": [{'prefix': '238.2.2.6'}, {'source': '192.168.0.1', 'prefix': '238.2.2.5'}]}
updates:
    description: commands sent to the device
    returned: always
    type: list
    sample: ["interface Ethernet1/32", "ip igmp startup-query-count 30",
             "ip igmp static-oif 238.2.2.6", "ip igmp static-oif 238.2.2.5 source 192.168.0.1"]
changed:
    description: check to see if a change was made on the device
    returned: always
    type: bool
    sample: true
"""

import re

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.nxos import (
    get_interface_type,
    load_config,
    run_commands,
)


def execute_show_command(command, module, command_type="cli_show"):
    if command_type == "cli_show_ascii":
        cmds = [{"command": command, "output": "text"}]
    else:
        cmds = [{"command": command, "output": "json"}]

    return run_commands(module, cmds)


def get_interface_mode(interface, intf_type, module):
    command = "show interface {0}".format(interface)
    interface = {}
    mode = "unknown"

    if intf_type in ["ethernet", "portchannel"]:
        body = execute_show_command(command, module)[0]
        interface_table = body["TABLE_interface"]["ROW_interface"]
        mode = str(interface_table.get("eth_mode", "layer3"))
        if mode == "access" or mode == "trunk":
            mode = "layer2"
    elif intf_type == "loopback" or intf_type == "svi":
        mode = "layer3"
    return mode


def apply_key_map(key_map, table):
    new_dict = {}
    for key, value in table.items():
        new_key = key_map.get(key)
        if new_key:
            value = table.get(key)
            if value:
                new_dict[new_key] = value
            else:
                new_dict[new_key] = value
    return new_dict


def flatten_list(command_lists):
    flat_command_list = []
    for command in command_lists:
        if isinstance(command, list):
            flat_command_list.extend(command)
        else:
            flat_command_list.append(command)
    return flat_command_list


def get_igmp_interface(module, interface):
    command = "show ip igmp interface {0}".format(interface)
    igmp = {}

    key_map = {
        "IGMPVersion": "version",
        "ConfiguredStartupQueryInterval": "startup_query_interval",
        "StartupQueryCount": "startup_query_count",
        "RobustnessVariable": "robustness",
        "ConfiguredQuerierTimeout": "querier_timeout",
        "ConfiguredMaxResponseTime": "query_mrt",
        "ConfiguredQueryInterval": "query_interval",
        "LastMemberMTR": "last_member_qrt",
        "LastMemberQueryCount": "last_member_query_count",
        "ConfiguredGroupTimeout": "group_timeout",
    }

    body = execute_show_command(command, module)[0]

    if body:
        if "not running" in body:
            return igmp
        resource = body["TABLE_vrf"]["ROW_vrf"]["TABLE_if"]["ROW_if"]
        igmp = apply_key_map(key_map, resource)
        report_llg = str(resource["ReportingForLinkLocal"]).lower()
        if report_llg == "true":
            igmp["report_llg"] = True
        elif report_llg == "false":
            igmp["report_llg"] = False

        immediate_leave = str(resource["ImmediateLeave"]).lower()  # returns en or dis
        if re.search(r"^en|^true|^enabled", immediate_leave):
            igmp["immediate_leave"] = True
        elif re.search(r"^dis|^false|^disabled", immediate_leave):
            igmp["immediate_leave"] = False

    # the  next block of code is used to retrieve anything with:
    # ip igmp static-oif *** i.e.. could be route-map ROUTEMAP
    # or PREFIX source <ip>, etc.
    command = "show run interface {0} | inc oif".format(interface)

    body = execute_show_command(command, module, command_type="cli_show_ascii")[0]

    staticoif = []
    if body:
        split_body = body.split("\n")
        route_map_regex = r".*ip igmp static-oif route-map\s+(?P<route_map>\S+).*"
        prefix_source_regex = (
            r".*ip igmp static-oif\s+(?P<prefix>"
            r"((\d+.){3}\d+))(\ssource\s"
            r"(?P<source>\S+))?.*"
        )

        for line in split_body:
            temp = {}
            try:
                match_route_map = re.match(route_map_regex, line, re.DOTALL)
                route_map = match_route_map.groupdict()["route_map"]
            except AttributeError:
                route_map = ""

            try:
                match_prefix_source = re.match(prefix_source_regex, line, re.DOTALL)
                prefix_source_group = match_prefix_source.groupdict()
                prefix = prefix_source_group["prefix"]
                source = prefix_source_group["source"]
            except AttributeError:
                prefix = ""
                source = ""

            if route_map:
                temp["route_map"] = route_map
            if prefix:
                temp["prefix"] = prefix
            if source:
                temp["source"] = source
            if temp:
                staticoif.append(temp)

    igmp["oif_routemap"] = None
    igmp["oif_prefix_source"] = []

    if staticoif:
        if len(staticoif) == 1 and staticoif[0].get("route_map"):
            igmp["oif_routemap"] = staticoif[0]["route_map"]
        else:
            igmp["oif_prefix_source"] = staticoif

    return igmp


def config_igmp_interface(delta, existing, existing_oif_prefix_source):
    CMDS = {
        "version": "ip igmp version {0}",
        "startup_query_interval": "ip igmp startup-query-interval {0}",
        "startup_query_count": "ip igmp startup-query-count {0}",
        "robustness": "ip igmp robustness-variable {0}",
        "querier_timeout": "ip igmp querier-timeout {0}",
        "query_mrt": "ip igmp query-max-response-time {0}",
        "query_interval": "ip igmp query-interval {0}",
        "last_member_qrt": "ip igmp last-member-query-response-time {0}",
        "last_member_query_count": "ip igmp last-member-query-count {0}",
        "group_timeout": "ip igmp group-timeout {0}",
        "report_llg": "ip igmp report-link-local-groups",
        "immediate_leave": "ip igmp immediate-leave",
        "oif_prefix_source": "ip igmp static-oif {0} source {1} ",
        "oif_routemap": "ip igmp static-oif route-map {0}",
        "oif_prefix": "ip igmp static-oif {0}",
    }

    commands = []
    command = None
    def_vals = get_igmp_interface_defaults()

    for key, value in delta.items():
        if key == "oif_ps" and value != "default":
            for each in value:
                if each in existing_oif_prefix_source:
                    existing_oif_prefix_source.remove(each)
                else:
                    # add new prefix/sources
                    pf = each["prefix"]
                    src = ""
                    if "source" in each.keys():
                        src = each["source"]
                    if src:
                        commands.append(CMDS.get("oif_prefix_source").format(pf, src))
                    else:
                        commands.append(CMDS.get("oif_prefix").format(pf))
            if existing_oif_prefix_source:
                for each in existing_oif_prefix_source:
                    # remove stale prefix/sources
                    pf = each["prefix"]
                    src = ""
                    if "source" in each.keys():
                        src = each["source"]
                    if src:
                        commands.append(
                            "no " + CMDS.get("oif_prefix_source").format(pf, src),
                        )
                    else:
                        commands.append("no " + CMDS.get("oif_prefix").format(pf))
        elif key == "oif_routemap":
            if value == "default":
                if existing.get(key):
                    command = "no " + CMDS.get(key).format(existing.get(key))
            else:
                command = CMDS.get(key).format(value)
        elif value:
            if value == "default":
                if def_vals.get(key) != existing.get(key):
                    command = CMDS.get(key).format(def_vals.get(key))
            else:
                command = CMDS.get(key).format(value)
        elif not value:
            command = "no {0}".format(CMDS.get(key).format(value))

        if command:
            if command not in commands:
                commands.append(command)
        command = None

    return commands


def get_igmp_interface_defaults():
    version = "2"
    startup_query_interval = "31"
    startup_query_count = "2"
    robustness = "2"
    querier_timeout = "255"
    query_mrt = "10"
    query_interval = "125"
    last_member_qrt = "1"
    last_member_query_count = "2"
    group_timeout = "260"
    report_llg = False
    immediate_leave = False

    args = dict(
        version=version,
        startup_query_interval=startup_query_interval,
        startup_query_count=startup_query_count,
        robustness=robustness,
        querier_timeout=querier_timeout,
        query_mrt=query_mrt,
        query_interval=query_interval,
        last_member_qrt=last_member_qrt,
        last_member_query_count=last_member_query_count,
        group_timeout=group_timeout,
        report_llg=report_llg,
        immediate_leave=immediate_leave,
    )

    default = dict((param, value) for (param, value) in args.items() if value is not None)

    return default


def config_default_igmp_interface(existing, delta):
    commands = []
    proposed = get_igmp_interface_defaults()
    delta = dict(set(proposed.items()).difference(existing.items()))
    if delta:
        command = config_igmp_interface(
            delta,
            existing,
            existing_oif_prefix_source=None,
        )

        if command:
            for each in command:
                commands.append(each)

    return commands


def config_remove_oif(existing, existing_oif_prefix_source):
    commands = []
    command = None
    if existing.get("oif_routemap"):
        commands.append(
            "no ip igmp static-oif route-map {0}".format(existing.get("oif_routemap")),
        )
    elif existing_oif_prefix_source:
        for each in existing_oif_prefix_source:
            if each.get("prefix") and each.get("source"):
                command = "no ip igmp static-oif {0} source {1} ".format(
                    each.get("prefix"),
                    each.get("source"),
                )
            elif each.get("prefix"):
                command = "no ip igmp static-oif {0}".format(each.get("prefix"))
            if command:
                commands.append(command)
            command = None

    return commands


def main():
    argument_spec = dict(
        interface=dict(required=True, type="str"),
        version=dict(required=False, type="str", choices=["2", "3", "default"]),
        startup_query_interval=dict(required=False, type="str"),
        startup_query_count=dict(required=False, type="str"),
        robustness=dict(required=False, type="str"),
        querier_timeout=dict(required=False, type="str"),
        query_mrt=dict(required=False, type="str"),
        query_interval=dict(required=False, type="str"),
        last_member_qrt=dict(required=False, type="str"),
        last_member_query_count=dict(required=False, type="str"),
        group_timeout=dict(required=False, type="str"),
        report_llg=dict(type="bool"),
        immediate_leave=dict(type="bool"),
        oif_routemap=dict(required=False, type="str"),
        oif_ps=dict(required=False, type="raw"),
        restart=dict(type="bool", default=False),
        state=dict(choices=["present", "absent", "default"], default="present"),
    )

    mutually_exclusive = [("oif_ps", "oif_routemap")]

    module = AnsibleModule(
        argument_spec=argument_spec,
        mutually_exclusive=mutually_exclusive,
        supports_check_mode=True,
    )

    warnings = list()

    state = module.params["state"]
    interface = module.params["interface"]
    oif_routemap = module.params["oif_routemap"]
    oif_ps = module.params["oif_ps"]

    intf_type = get_interface_type(interface)
    if get_interface_mode(interface, intf_type, module) == "layer2":
        module.fail_json(msg="this module only works on Layer 3 interfaces")

    existing = get_igmp_interface(module, interface)
    existing_copy = existing.copy()
    end_state = existing_copy

    if not existing.get("version"):
        module.fail_json(msg="pim needs to be enabled on the interface")

    existing_oif_prefix_source = existing.get("oif_prefix_source")
    # not json serializable
    existing.pop("oif_prefix_source")

    if oif_routemap and existing_oif_prefix_source:
        module.fail_json(
            msg="Delete static-oif configurations on this "
            "interface if you want to use a routemap",
        )

    if oif_ps and existing.get("oif_routemap"):
        module.fail_json(
            msg="Delete static-oif route-map configuration "
            "on this interface if you want to config "
            "static entries",
        )

    args = [
        "version",
        "startup_query_interval",
        "startup_query_count",
        "robustness",
        "querier_timeout",
        "query_mrt",
        "query_interval",
        "last_member_qrt",
        "last_member_query_count",
        "group_timeout",
        "report_llg",
        "immediate_leave",
        "oif_routemap",
    ]

    changed = False
    commands = []
    proposed = dict((k, v) for k, v in module.params.items() if v is not None and k in args)

    CANNOT_ABSENT = [
        "version",
        "startup_query_interval",
        "startup_query_count",
        "robustness",
        "querier_timeout",
        "query_mrt",
        "query_interval",
        "last_member_qrt",
        "last_member_query_count",
        "group_timeout",
        "report_llg",
        "immediate_leave",
    ]

    if state == "absent":
        for each in CANNOT_ABSENT:
            if each in proposed:
                module.fail_json(
                    msg="only params: " "oif_ps, oif_routemap can be used when " "state=absent",
                )

    # delta check for all params except oif_ps
    delta = dict(set(proposed.items()).difference(existing.items()))

    if oif_ps:
        if oif_ps == "default":
            delta["oif_ps"] = []
        else:
            delta["oif_ps"] = oif_ps

    if state == "present":
        if delta:
            command = config_igmp_interface(delta, existing, existing_oif_prefix_source)
            if command:
                commands.append(command)

    elif state == "default":
        command = config_default_igmp_interface(existing, delta)
        if command:
            commands.append(command)
    elif state == "absent":
        command = None
        if existing.get("oif_routemap") or existing_oif_prefix_source:
            command = config_remove_oif(existing, existing_oif_prefix_source)

        if command:
            commands.append(command)

        command = config_default_igmp_interface(existing, delta)
        if command:
            commands.append(command)

    cmds = []
    results = {}
    if commands:
        commands.insert(0, ["interface {0}".format(interface)])
        cmds = flatten_list(commands)

        if module.check_mode:
            module.exit_json(changed=True, commands=cmds)
        else:
            load_config(module, cmds)
            changed = True
            end_state = get_igmp_interface(module, interface)
            if "configure" in cmds:
                cmds.pop(0)

    if module.params["restart"]:
        cmd = {"command": "restart igmp", "output": "text"}
        run_commands(module, cmd)

    results["proposed"] = proposed
    results["existing"] = existing_copy
    results["updates"] = cmds
    results["changed"] = changed
    results["warnings"] = warnings
    results["end_state"] = end_state

    module.exit_json(**results)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The Ntp_global parser templates file. This contains
a list of parser definitions and associated functions that
facilitates both facts gathering and native command generation for
the given network resource.
"""

import re

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.network_template import (
    NetworkTemplate,
)


class Ntp_globalTemplate(NetworkTemplate):
    def __init__(self, lines=None, module=None):
        super(Ntp_globalTemplate, self).__init__(lines=lines, tmplt=self, module=module)

    # fmt: off
    PARSERS = [
        {
            "name": "access_group.match_all",
            "getval": re.compile(
                r"""
                ^ntp\saccess-group\s(?P<match_all>match-all)
                $""", re.VERBOSE,
            ),
            "setval": "ntp access-group match-all",
            "result": {
                "access_group": {
                    "match_all": "{{ True if match_all is defined else None }}",
                },
            },
        },
        {
            "name": "peer",
            "getval": re.compile(
                r"""
                ^ntp\saccess-group\speer\s(?P<acl>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "ntp access-group peer {{ access_list }}",
            "result": {
                "access_group": {
                    "peer": [
                        {
                            "access_list": "{{ acl }}",
                        },
                    ],
                },
            },
        },
        {
            "name": "query_only",
            "getval": re.compile(
                r"""
                ^ntp\saccess-group\squery-only\s(?P<acl>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "ntp access-group query-only {{ access_list }}",
            "result": {
                "access_group": {
                    "query_only": [
                        {
                            "access_list": "{{ acl }}",
                        },
                    ],
                },
            },
        },
        {
            "name": "serve",
            "getval": re.compile(
                r"""
                ^ntp\saccess-group\sserve\s(?P<acl>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "ntp access-group serve {{ access_list }}",
            "result": {
                "access_group": {
                    "serve": [
                        {
                            "access_list": "{{ acl }}",
                        },
                    ],
                },
            },
        },
        {
            "name": "serve_only",
            "getval": re.compile(
                r"""
                ^ntp\saccess-group\sserve-only\s(?P<acl>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "ntp access-group serve-only {{ access_list }}",
            "result": {
                "access_group": {
                    "serve_only": [
                        {
                            "access_list": "{{ acl }}",
                        },
                    ],
                },
            },
        },
        {
            "name": "allow.control.rate_limit",
            "getval": re.compile(
                r"""
                ^ntp\sallow\scontrol\srate-limit\s(?P<rate_limit>\d+)
                $""", re.VERBOSE,
            ),
            "setval": "ntp allow control rate-limit {{ allow.control.rate_limit }}",
            "result": {
                "allow": {
                    "control": {
                        "rate_limit": "{{ rate_limit }}",
                    },
                },
            },
        },
        {
            "name": "allow.private",
            "getval": re.compile(
                r"""
                ^ntp\sallow\s(?P<private>private)
                $""", re.VERBOSE,
            ),
            "setval": "ntp allow private",
            "result": {
                "allow": {
                    "private": "{{ not not private }}",
                },
            },
        },
        {
            "name": "authenticate",
            "getval": re.compile(
                r"""
                ^ntp\s(?P<authenticate>authenticate)
                $""", re.VERBOSE,
            ),
            "setval": "ntp authenticate",
            "result": {
                "authenticate": "{{ not not authenticate }}",
            },
        },
        {
            "name": "authentication_keys",
            "getval": re.compile(
                r"""
                ^ntp\sauthentication-key\s(?P<id>\d+)\smd5\s(?P<key>\S+)\s(?P<encryption>\d+)
                $""", re.VERBOSE,
            ),
            "setval": "ntp authentication-key {{ id }} md5 {{ key }} {{ encryption }}",
            "result": {
                "authentication_keys": [
                    {
                        "id": "{{ id }}",
                        "key": "{{ key }}",
                        "encryption": "{{ encryption }}",
                    },
                ],
            },
        },
        {
            "name": "logging",
            "getval": re.compile(
                r"""
                ^ntp\s(?P<logging>logging)
                $""", re.VERBOSE,
            ),
            "setval": "ntp logging",
            "result": {
                "logging": "{{ not not logging }}",
            },
        },
        {
            "name": "master.stratum",
            "getval": re.compile(
                r"""
                ^ntp\smaster\s(?P<stratum>\d+)
                $""", re.VERBOSE,
            ),
            "setval": "ntp master {{ master.stratum }}",
            "result": {
                "master": {
                    "stratum": "{{ stratum }}",
                },
            },
        },
        {
            "name": "passive",
            "getval": re.compile(
                r"""
                ^ntp\s(?P<passive>passive)
                $""", re.VERBOSE,
            ),
            "setval": "ntp passive",
            "result": {
                "passive": "{{ not not passive }}",
            },
        },
        {
            "name": "peers",
            "getval": re.compile(
                r"""
                ^ntp\speer
                \s(?P<peer>\S+)
                (\s(?P<prefer>prefer))?
                (\suse-vrf\s(?P<use_vrf>\S+))?
                (\skey\s(?P<key>\d+))?
                (\sminpoll\s(?P<minpoll>\d+))?
                (\smaxpoll\s(?P<maxpoll>\d+))?
                $""", re.VERBOSE,
            ),
            "setval": "ntp peer {{ peer }}"
                      "{{ ' prefer' if prefer|d(False) else ''}}"
                      "{{ (' use-vrf ' + vrf) if vrf is defined else '' }}"
                      "{{ (' key ' + key_id|string) if key_id is defined else '' }}"
                      "{{ (' minpoll ' + minpoll|string) if minpoll is defined else '' }}"
                      "{{ (' maxpoll ' + maxpoll|string) if maxpoll is defined else '' }}",
            "result": {
                "peers": [
                    {
                        "peer": "{{ peer }}",
                        "prefer": "{{ not not prefer }}",
                        "vrf": "{{ use_vrf }}",
                        "key_id": "{{ key }}",
                        "minpoll": "{{ minpoll }}",
                        "maxpoll": "{{ maxpoll }}",
                    },
                ],
            },
        },
        {
            "name": "servers",
            "getval": re.compile(
                r"""
                ^ntp\sserver
                \s(?P<server>\S+)
                (\s(?P<prefer>prefer))?
                (\suse-vrf\s(?P<use_vrf>\S+))?
                (\skey\s(?P<key>\d+))?
                (\sminpoll\s(?P<minpoll>\d+))?
                (\smaxpoll\s(?P<maxpoll>\d+))?
                $""", re.VERBOSE,
            ),
            "setval": "ntp server {{ server }}"
                      "{{ ' prefer' if prefer|d(False) else ''}}"
                      "{{ (' use-vrf ' + vrf) if vrf is defined else '' }}"
                      "{{ (' key ' + key_id|string) if key_id is defined else '' }}"
                      "{{ (' minpoll ' + minpoll|string) if minpoll is defined else '' }}"
                      "{{ (' maxpoll ' + maxpoll|string) if maxpoll is defined else '' }}",
            "result": {
                "servers": [
                    {
                        "server": "{{ server }}",
                        "prefer": "{{ not not prefer }}",
                        "vrf": "{{ use_vrf }}",
                        "key_id": "{{ key }}",
                        "minpoll": "{{ minpoll }}",
                        "maxpoll": "{{ maxpoll }}",
                    },
                ],
            },
        },
        {
            "name": "source",
            "getval": re.compile(
                r"""
                ^ntp\ssource\s(?P<source>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "ntp source {{ source }}",
            "result": {
                "source": "{{ source }}",
            },
        },
        {
            "name": "source_interface",
            "getval": re.compile(
                r"""
                ^ntp\ssource-interface(\s)+(?P<source_interface>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "ntp source-interface {{ source_interface }}",
            "result": {
                "source_interface": "{{ source_interface }}",
            },
        },
        {
            "name": "trusted_keys",
            "getval": re.compile(
                r"""
                ^ntp\strusted-key\s(?P<key>\d+)
                $""", re.VERBOSE,
            ),
            "setval": "ntp trusted-key {{ key_id|string }}",
            "result": {
                "trusted_keys": [
                    {
                        "key_id": "{{ key }}",
                    },
                ],
            },
        },
    ]
    # fmt: on



# Copyright (c) 2023, Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
name: onepassword_doc
author:
  - Sam Doran (@samdoran)
short_description: Fetch documents stored in 1Password
version_added: "8.1.0"
description:
  - P(community.general.onepassword_doc#lookup) wraps C(op) command line utility to fetch one or more documents from 1Password.
notes:
  - The document contents are a string exactly as stored in 1Password.
options:
  _terms:
    description: Identifier(s) (case-insensitive UUID or name) of item(s) to retrieve.
    required: true
    type: list
    elements: string

extends_documentation_fragment:
  - community.general._onepassword
  - community.general._onepassword.lookup
"""

EXAMPLES = r"""
---
- name: Retrieve a private key from 1Password
  ansible.builtin.debug:
    var: lookup('community.general.onepassword_doc', 'Private key')
"""

RETURN = r"""
_raw:
  description: Requested document.
  type: list
  elements: string
"""

from ansible.plugins.lookup import LookupBase

from ansible_collections.community.general.plugins.lookup.onepassword import OnePass, OnePassCLIv2
from ansible_collections.community.general.plugins.plugin_utils._lookup import check_for_wrong_terms


class OnePassCLIv2Doc(OnePassCLIv2):
    def get_raw(self, item_id, vault=None, token=None):
        args = ["document", "get", item_id]
        return self._add_parameters_and_run(args, vault=vault, token=token)


class LookupModule(LookupBase):
    def run(self, terms, variables=None, **kwargs):
        self.set_options(var_options=variables, direct=kwargs)
        check_for_wrong_terms(self, direct=kwargs)

        vault = self.get_option("vault")
        subdomain = self.get_option("subdomain")
        domain = self.get_option("domain", "1password.com")
        username = self.get_option("username")
        secret_key = self.get_option("secret_key")
        master_password = self.get_option("master_password")
        service_account_token = self.get_option("service_account_token")
        account_id = self.get_option("account_id")
        connect_host = self.get_option("connect_host")
        connect_token = self.get_option("connect_token")

        op = OnePass(
            subdomain=subdomain,
            domain=domain,
            username=username,
            secret_key=secret_key,
            master_password=master_password,
            service_account_token=service_account_token,
            account_id=account_id,
            connect_host=connect_host,
            connect_token=connect_token,
            cli_class=OnePassCLIv2Doc,
        )
        op.assert_logged_in()

        values = []
        for term in terms:
            values.append(op.get_raw(term, vault))

        return values



#
# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
The nxos static_routes fact class
It is in this file the configuration is collected from the device
for a given resource, parsed, and the facts tree is populated
based on the configuration.
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common import utils

from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.argspec.static_routes.static_routes import (
    Static_routesArgs,
)
from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.rm_templates.static_routes import (
    Static_routesTemplate,
)


class Static_routesFacts(object):
    """The nxos static_routes fact class"""

    def __init__(self, module):
        self._module = module
        self.argument_spec = Static_routesArgs.argument_spec

    def get_static_routes_data(self, connection):
        non_vrf_data = connection.get("show running-config | include '^ip(v6)* route'") or ""
        if non_vrf_data:
            non_vrf_data = "\n".join(
                line for line in non_vrf_data.splitlines() if line and not line.startswith("  ")
            )
        vrf_data = connection.get("show running-config | section '^vrf context'")
        if vrf_data:
            non_vrf_data += "\n" + vrf_data
        return non_vrf_data

    def process_static_routes(self, objs):
        strout = {}
        for k, obj in objs.items():
            _routes = {"next_hops": []}
            _nx_hop = []
            is_vrf = False

            for routes in obj:
                grp_vrf = routes.pop("_vrf", None)
                if grp_vrf:
                    is_vrf = True
                _afi = routes.pop("_afi")

                _routes["dest"] = routes.pop("_dest")
                _nx_hop.append(routes)

            _routes["next_hops"].extend(_nx_hop)

            if is_vrf:
                if strout.get(grp_vrf) and strout[grp_vrf].get(_afi):
                    strout[grp_vrf][_afi].append(_routes)
                else:
                    if strout.get(grp_vrf):
                        _tma = {_afi: [_routes]}
                        strout[grp_vrf].update(_tma)
                    else:
                        _tm = {grp_vrf: {_afi: [_routes]}}
                        strout.update(_tm)
            else:
                if strout.get(_afi):
                    strout[_afi].append(_routes)
                else:
                    _tma = {_afi: [_routes]}
                    strout.update(_tma)
        return strout

    def structure_static_routes(self, strout):
        _static_route_facts = []
        afi_v4 = strout.pop("ipv4", None)
        afi_v6 = strout.pop("ipv6", None)

        if afi_v4 or afi_v6:
            _triv_static_route = {"address_families": []}

            if afi_v4:
                _triv_static_route["address_families"].append(
                    {"afi": "ipv4", "routes": afi_v4},
                )
            if afi_v6:
                _triv_static_route["address_families"].append(
                    {"afi": "ipv6", "routes": afi_v6},
                )

            _static_route_facts.append(_triv_static_route)

        for k, v in strout.items():
            afi_v4 = v.pop("ipv4", None)
            afi_v6 = v.pop("ipv6", None)

            _vrf_static_route = {
                "vrf": k,
                "address_families": [],
            }

            if afi_v4:
                _vrf_static_route["address_families"].append(
                    {"afi": "ipv4", "routes": afi_v4},
                )
            if afi_v6:
                _vrf_static_route["address_families"].append(
                    {"afi": "ipv6", "routes": afi_v6},
                )

            _static_route_facts.append(_vrf_static_route)
        return _static_route_facts

    def populate_facts(self, connection, ansible_facts, data=None):
        """Populate the facts for Static_routes network resource

        :param connection: the device connection
        :param ansible_facts: Facts dictionary
        :param data: previously collected conf

        :rtype: dictionary
        :returns: facts
        """

        facts = {}
        objs = []

        if not data:
            data = self.get_static_routes_data(connection)

        # parse native config using the Static_routes template
        static_routes_parser = Static_routesTemplate(
            lines=data.splitlines(),
            module=self._module,
        )
        objs = static_routes_parser.parse()

        strout = self.process_static_routes(objs)
        objs = self.structure_static_routes(strout)

        ansible_facts["ansible_network_resources"].pop("static_routes", None)

        params = utils.remove_empties(
            static_routes_parser.validate_config(
                self.argument_spec,
                {"config": objs},
                redact=True,
            ),
        )

        facts["static_routes"] = params.get("config")
        ansible_facts["ansible_network_resources"].update(facts)

        return ansible_facts



# -*- coding: utf-8 -*-
# Copyright 2025 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The Bfd_interfaces parser templates file. This contains
a list of parser definitions and associated functions that
facilitates both facts gathering and native command generation for
the given network resource.
"""

import re

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.network_template import (
    NetworkTemplate,
)


class Bfd_interfacesTemplate(NetworkTemplate):
    def __init__(self, lines=None, module=None):
        super(Bfd_interfacesTemplate, self).__init__(lines=lines, tmplt=self, module=module)

    # fmt: off
    PARSERS = [
        {
            "name": "name",
            "getval": re.compile(
                r"""^interface
                    (\s(?P<name>\S+))
                    $""",
                re.VERBOSE,
            ),
            "compval": "name",
            "setval": "interface {{ name }}",
            "result": {"{{ name }}": {"name": "{{ name }}"}},
            "shared": True,
        },
        {
            "name": "bfd",
            "getval": re.compile(
                r"""
                (\s+(?P<no>no))
                \s+bfd\senable
                $""", re.VERBOSE,
            ),
            "setval": "bfd enable",
            "result": {
                "{{ name }}": {
                    "bfd": "{{ False if no is defined else True }}",
                },
            },
        },
        {
            "name": "echo",
            "getval": re.compile(
                r"""
                (\s+(?P<no>no))
                \s+bfd\secho
                $""", re.VERBOSE,
            ),
            "setval": "bfd echo",
            "result": {
                "{{ name }}": {
                    "echo": "{{ False if no is defined else True }}",
                },
            },
        },
        {
            "name": "jitter",
            "getval": re.compile(
                r"""
                (\s+(?P<no>no))
                \s+bfd\sjitter
                $""", re.VERBOSE,
            ),
            "setval": "bfd jitter",
            "result": {
                "{{ name }}": {
                    "jitter": "{{ False if no is defined else True }}",
                },
            },
        },
        {
            "name": "local_address",
            "getval": re.compile(
                r"""\s+bfd\slocal-address
                    (\s(?P<local_address>\S+))
                    $""",
                re.VERBOSE,
            ),
            "setval": "bfd local-address {{ local_address }}",
            "result": {"{{ name }}": {"local_address": "{{ local_address }}"}},
        },
        {
            "name": "template",
            "getval": re.compile(
                r"""\s+bfd\stemplate
                    (\s(?P<template>\S+))
                    $""",
                re.VERBOSE,
            ),
            "setval": "bfd template {{ template }}",
            "result": {"{{ name }}": {"template": "{{ template }}"}},
        },
        {
            "name": "interval",
            "getval": re.compile(
                r"""
                \s+bfd\sinterval
                \s+(?P<input>\d+)
                \s+min_rx\s(?P<min_rx>\d+)
                \s+multiplier\s(?P<multiplier>\d+)
                $""",
                re.VERBOSE,
            ),
            "setval": "bfd interval {{ interval.input }} "
                      "min_rx {{ interval.min_rx }} "
                      "multiplier {{ interval.multiplier }}",
            "result": {
                "{{ name }}": {
                    "interval": {
                        "input": "{{ input }}",
                        "min_rx": "{{ min_rx | int }}",
                        "multiplier": "{{ multiplier }}",
                    },
                },
            },
        },
    ]
    # fmt: on
